@@ -5,36 +5,28 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.catalina.ant.jmx;
 
-/**
- *
- * @author Peter Rossbach
- * @version $Revision: 467222 $ $Date: 2006-10-24 05:17:11 +0200 (Tue, 24 Oct 2006) $
- * @since 5.5.10
- */
 public class Arg {
-    String type;
-    String value;
 
-    public void setType( String type) {
-        this.type=type;
+    private String type;
+    private String value;
+
+    public void setType(String type) {
+        this.type = type;
     }
-    public void setValue( String value ) {
-        this.value=value;
-    }
-    public void addText( String text ) {
-        this.value=text;
+
+    public void setValue(String value) {
+        this.value = value;
     }
 
     public String getValue() {
