@@ -243,11 +243,7 @@ public class OutputBuffer extends Writer {
             coyoteResponse.setContentLength(bb.remaining());
         }
 
-        if (coyoteResponse.getStatus() == HttpServletResponse.SC_SWITCHING_PROTOCOLS) {
-            doFlush(true);
-        } else {
-            doFlush(false);
-        }
+        doFlush(coyoteResponse.getStatus() == HttpServletResponse.SC_SWITCHING_PROTOCOLS);
         closed = true;
 
         // The request should have been completely read by the time the response
@@ -333,24 +329,24 @@ public class OutputBuffer extends Writer {
             try {
                 coyoteResponse.doWrite(buf);
             } catch (CloseNowException e) {
-                // Catch this sub-class as it requires specific handling.
+                // Catch this subclass as it requires specific handling.
                 // Examples where this exception is thrown:
                 // - HTTP/2 stream timeout
                 // Prevent further output for this response
                 closed = true;
                 throw e;
-            } catch (IOException e) {
+            } catch (IOException ioe) {
                 // An IOException on a write is almost always due to
                 // the remote client aborting the request. Wrap this
                 // so that it can be handled better by the error dispatcher.
-                throw new ClientAbortException(e);
+                throw new ClientAbortException(ioe);
             }
         }
 
     }
 
 
-    public void write(byte b[], int off, int len) throws IOException {
+    public void write(byte[] b, int off, int len) throws IOException {
 
         if (suspended) {
             return;
@@ -372,7 +368,7 @@ public class OutputBuffer extends Writer {
     }
 
 
-    private void writeBytes(byte b[], int off, int len) throws IOException {
+    private void writeBytes(byte[] b, int off, int len) throws IOException {
 
         if (closed) {
             return;
@@ -445,12 +441,12 @@ public class OutputBuffer extends Writer {
             }
             if (from.remaining() > 0) {
                 flushByteBuffer();
-            } else if (conv.isUndeflow() && bb.limit() > bb.capacity() - 4) {
+            } else if (conv.isUnderflow() && bb.limit() > bb.capacity() - 4) {
                 // Handle an edge case. There are no more chars to write at the
                 // moment but there is a leftover character in the converter
                 // which must be part of a surrogate pair. The byte buffer does
                 // not have enough space left to output the bytes for this pair
-                // once it is complete )it will require 4 bytes) so flush now to
+                // once it is complete (it will require 4 bytes) so flush now to
                 // prevent the bytes for the leftover char and the rest of the
                 // surrogate pair yet to be written from being lost.
                 // See TestOutputBuffer#testUtf8SurrogateBody()
@@ -478,7 +474,7 @@ public class OutputBuffer extends Writer {
 
 
     @Override
-    public void write(char c[]) throws IOException {
+    public void write(char[] c) throws IOException {
 
         if (suspended) {
             return;
@@ -490,7 +486,7 @@ public class OutputBuffer extends Writer {
 
 
     @Override
-    public void write(char c[], int off, int len) throws IOException {
+    public void write(char[] c, int off, int len) throws IOException {
 
         if (suspended) {
             return;
@@ -667,7 +663,7 @@ public class OutputBuffer extends Writer {
      *
      * @throws IOException Writing overflow data to the output channel failed
      */
-    public void append(byte src[], int off, int len) throws IOException {
+    public void append(byte[] src, int off, int len) throws IOException {
         if (bb.remaining() == 0) {
             appendByteArray(src, off, len);
         } else {
@@ -690,7 +686,7 @@ public class OutputBuffer extends Writer {
      *
      * @throws IOException Writing overflow data to the output channel failed
      */
-    public void append(char src[], int off, int len) throws IOException {
+    public void append(char[] src, int off, int len) throws IOException {
         // if we have limit and we're below
         if (len <= cb.capacity() - cb.limit()) {
             transfer(src, off, len, cb);
@@ -741,7 +737,7 @@ public class OutputBuffer extends Writer {
     }
 
 
-    private void appendByteArray(byte src[], int off, int len) throws IOException {
+    private void appendByteArray(byte[] src, int off, int len) throws IOException {
         if (len == 0) {
             return;
         }
