@@ -1483,8 +1483,8 @@ public class Response implements HttpServletResponse {
                 redirectURLCC.append(':');
                 redirectURLCC.append(location, 0, location.length());
                 return redirectURLCC.toString();
-            } catch (IOException e) {
-                throw new IllegalArgumentException(location, e);
+            } catch (IOException ioe) {
+                throw new IllegalArgumentException(location, ioe);
             }
 
         } else if (leadingSlash || !UriUtil.hasScheme(location)) {
@@ -1525,8 +1525,8 @@ public class Response implements HttpServletResponse {
                 redirectURLCC.append(location, 0, location.length());
 
                 normalize(redirectURLCC);
-            } catch (IOException e) {
-                throw new IllegalArgumentException(location, e);
+            } catch (IOException ioe) {
+                throw new IllegalArgumentException(location, ioe);
             }
 
             return redirectURLCC.toString();
@@ -1560,8 +1560,8 @@ public class Response implements HttpServletResponse {
         if (cc.endsWith("/.") || cc.endsWith("/..")) {
             try {
                 cc.append('/');
-            } catch (IOException e) {
-                throw new IllegalArgumentException(cc.toString(), e);
+            } catch (IOException ioe) {
+                throw new IllegalArgumentException(cc.toString(), ioe);
             }
         }
 
