@@ -74,7 +74,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
     /**
      * The set of attribute names that are special for request dispatchers.
      */
-    protected static final String specials[] =
+    protected static final String[] specials =
             { RequestDispatcher.INCLUDE_REQUEST_URI, RequestDispatcher.INCLUDE_CONTEXT_PATH,
                     RequestDispatcher.INCLUDE_SERVLET_PATH, RequestDispatcher.INCLUDE_PATH_INFO,
                     RequestDispatcher.INCLUDE_QUERY_STRING, RequestDispatcher.INCLUDE_MAPPING,
@@ -93,7 +93,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
     }
 
     private static final int shortestSpecialNameLength =
-            specialsMap.keySet().stream().mapToInt(s -> s.length()).min().getAsInt();
+            specialsMap.keySet().stream().mapToInt(String::length).min().getAsInt();
 
 
     private static final int SPECIALS_FIRST_FORWARD_INDEX = 6;
@@ -346,7 +346,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
 
         // Add the path info, if there is any
         String pathInfo = getPathInfo();
-        String requestPath = null;
+        String requestPath;
 
         if (pathInfo == null) {
             requestPath = servletPath;
@@ -355,7 +355,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
         }
 
         int pos = requestPath.lastIndexOf('/');
-        String relative = null;
+        String relative;
         if (context.getDispatchersUseEncodedPaths()) {
             if (pos >= 0) {
                 relative = URLEncoder.DEFAULT.encode(requestPath.substring(0, pos + 1), StandardCharsets.UTF_8) + path;
@@ -550,7 +550,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
                     if (localSession != null && !localSession.isValid()) {
                         localSession = null;
                     }
-                } catch (IOException e) {
+                } catch (IOException ignore) {
                     // Ignore
                 }
                 if (localSession == null && create) {
@@ -596,14 +596,10 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
             Session session = null;
             try {
                 session = manager.findSession(requestedSessionId);
-            } catch (IOException e) {
+            } catch (IOException ignore) {
                 // Ignore
             }
-            if ((session != null) && session.isValid()) {
-                return true;
-            } else {
-                return false;
-            }
+            return (session != null) && session.isValid();
 
         } else {
             return super.isRequestedSessionIdValid();
@@ -732,7 +728,12 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
             return;
         }
 
-        parameters = new ParameterMap<>(getRequest().getParameterMap());
+        Map<String,String[]> requestParameters = getRequest().getParameterMap();
+        if (requestParameters instanceof ParameterMap<?,?>) {
+            parameters = new ParameterMap<>((ParameterMap<String,String[]>) requestParameters);
+        } else {
+            parameters = new ParameterMap<>(requestParameters);
+        }
         mergeParameters();
         ((ParameterMap<String,String[]>) parameters).setLocked(true);
         parsedParams = true;
@@ -825,17 +826,13 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      */
     private String[] mergeValues(String[] values1, String[] values2) {
 
-        List<Object> results = new ArrayList<>();
+        List<String> results = new ArrayList<>();
 
-        if (values1 == null) {
-            // Skip - nothing to merge
-        } else {
+        if (values1 != null) {
             results.addAll(Arrays.asList(values1));
         }
 
-        if (values2 == null) {
-            // Skip - nothing to merge
-        } else {
+        if (values2 != null) {
             results.addAll(Arrays.asList(values2));
         }
 
@@ -852,7 +849,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      */
     private void mergeParameters() {
 
-        if ((queryParamString == null) || (queryParamString.length() < 1)) {
+        if ((queryParamString == null) || (queryParamString.isEmpty())) {
             return;
         }
 
