@@ -138,7 +138,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
     }
 
     /**
-     * Detect possible the JVMRoute change at cluster backup node..
+     * Detect possible the JVMRoute change at cluster backup node.
      *
      * @param request  tomcat request being processed
      * @param response tomcat response being processed
@@ -184,10 +184,10 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
                 return;
             }
             handleJvmRoute(request, sessionID, jvmRoute);
-            if (log.isDebugEnabled()) {
+            if (log.isTraceEnabled()) {
                 long t2 = System.currentTimeMillis();
                 long time = t2 - t1;
-                log.debug(sm.getString("jvmRoute.turnoverInfo", Long.valueOf(time)));
+                log.trace(sm.getString("jvmRoute.turnoverInfo", Long.valueOf(time)));
             }
         }
     }
@@ -218,7 +218,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
         Manager manager = request.getContext().getManager();
         if (log.isDebugEnabled()) {
             if (manager != null) {
-                log.debug(sm.getString("jvmRoute.foundManager", manager, request.getContext().getName()));
+                log.trace(sm.getString("jvmRoute.foundManager", manager, request.getContext().getName()));
             } else {
                 log.debug(sm.getString("jvmRoute.notFoundManager", request.getContext().getName()));
             }
@@ -226,17 +226,11 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
         return manager;
     }
 
-    /**
-     * @return Returns the cluster.
-     */
     @Override
     public CatalinaCluster getCluster() {
         return cluster;
     }
 
-    /**
-     * @param cluster The cluster to set.
-     */
     @Override
     public void setCluster(CatalinaCluster cluster) {
         this.cluster = cluster;
@@ -264,8 +258,8 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
             Session catalinaSession = null;
             try {
                 catalinaSession = getManager(request).findSession(sessionId);
-            } catch (IOException e) {
-                // Hups!
+            } catch (IOException ignore) {
+                // Error looking for session using old session ID. Treat it as not found.
             }
             String id = sessionId.substring(0, index);
             String newSessionID = id + "." + localJvmRoute;
@@ -276,11 +270,11 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
             } else {
                 try {
                     catalinaSession = getManager(request).findSession(newSessionID);
-                } catch (IOException e) {
-                    // Hups!
+                } catch (IOException ignore) {
+                    // Error looking for session using new session ID. Treat it as not found.
                 }
                 if (catalinaSession != null) {
-                    // session is rewrite at other request, rewrite this also
+                    // Session was rewritten in other, concurrent request. Rewrite this request also.
                     changeRequestSessionID(request, sessionId, newSessionID);
                 } else {
                     if (log.isDebugEnabled()) {
@@ -355,7 +349,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
      *                                   used
      */
     @Override
-    protected synchronized void startInternal() throws LifecycleException {
+    protected void startInternal() throws LifecycleException {
 
         if (cluster == null) {
             Cluster containerCluster = getContainer().getCluster();
@@ -383,7 +377,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
      *                                   used
      */
     @Override
-    protected synchronized void stopInternal() throws LifecycleException {
+    protected void stopInternal() throws LifecycleException {
 
         super.stopInternal();
 
