@@ -18,7 +18,6 @@ package org.apache.catalina.ha.tcp;
 
 import java.io.IOException;
 import java.util.ArrayList;
-import java.util.Iterator;
 import java.util.List;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.atomic.LongAdder;
@@ -101,7 +100,7 @@ public class ReplicationValve extends ValveBase implements ClusterValve {
     protected LongAdder nrOfCrossContextSendRequests = new LongAdder();
 
     /**
-     * must primary change indicator set
+     * Must set primary change indicator.
      */
     protected boolean primaryIndicator = false;
 
@@ -154,7 +153,7 @@ public class ReplicationValve extends ValveBase implements ClusterValve {
             log.trace(sm.getString("ReplicationValve.filter.loading", filter));
         }
 
-        if (filter == null || filter.length() == 0) {
+        if (filter == null || filter.isEmpty()) {
             this.filter = null;
         } else {
             try {
@@ -283,16 +282,6 @@ public class ReplicationValve extends ValveBase implements ClusterValve {
         }
     }
 
-    /**
-     * Log the interesting request parameters, invoke the next Valve in the sequence, and log the interesting response
-     * parameters.
-     *
-     * @param request  The servlet request to be processed
-     * @param response The servlet response to be created
-     *
-     * @exception IOException      if an input/output error occurs
-     * @exception ServletException if a servlet error occurs
-     */
     @Override
     public void invoke(Request request, Response response) throws IOException, ServletException {
         long totalstart = 0;
@@ -305,13 +294,13 @@ public class ReplicationValve extends ValveBase implements ClusterValve {
             createPrimaryIndicator(request);
         }
         Context context = request.getContext();
-        boolean isCrossContext = context != null && context instanceof StandardContext && context.getCrossContext();
+        boolean isCrossContext = context instanceof StandardContext && context.getCrossContext();
+        boolean isAsync = request.getAsyncContextInternal() != null;
         try {
             if (isCrossContext) {
                 if (log.isTraceEnabled()) {
                     log.trace(sm.getString("ReplicationValve.crossContext.add"));
                 }
-                // FIXME add Pool of Arraylists
                 crossContextSessions.set(new ArrayList<>());
             }
             getNext().invoke(request, response);
@@ -324,13 +313,13 @@ public class ReplicationValve extends ValveBase implements ClusterValve {
                     return;
                 }
                 if (cluster.hasMembers()) {
-                    sendReplicationMessage(request, totalstart, isCrossContext, clusterManager);
+                    sendReplicationMessage(request, totalstart, isCrossContext, isAsync, clusterManager);
                 } else {
                     resetReplicationRequest(request, isCrossContext);
                 }
             }
         } finally {
-            // Array must be remove: Current master request send endAccess at recycle.
+            // Array must be removed: Current master request send endAccess at recycle.
             // Don't register this request session again!
             if (isCrossContext) {
                 if (log.isTraceEnabled()) {
@@ -355,13 +344,6 @@ public class ReplicationValve extends ValveBase implements ClusterValve {
         nrOfCrossContextSendRequests.reset();
     }
 
-    /**
-     * Start this component and implement the requirements of
-     * {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
-     *
-     * @exception LifecycleException if this component detects a fatal error that prevents this component from being
-     *                                   used
-     */
     @Override
     protected void startInternal() throws LifecycleException {
         if (cluster == null) {
@@ -380,7 +362,7 @@ public class ReplicationValve extends ValveBase implements ClusterValve {
 
     // --------------------------------------------------------- Protected Methods
 
-    protected void sendReplicationMessage(Request request, long totalstart, boolean isCrossContext,
+    protected void sendReplicationMessage(Request request, long totalstart, boolean isCrossContext, boolean isAsync,
             ClusterManager clusterManager) {
         // this happens after the request
         long start = 0;
@@ -389,21 +371,18 @@ public class ReplicationValve extends ValveBase implements ClusterValve {
         }
         try {
             // send invalid sessions
-            // DeltaManager returns String[0]
-            if (!(clusterManager instanceof DeltaManager)) {
-                sendInvalidSessions(clusterManager);
-            }
+            sendInvalidSessions(clusterManager);
             // send replication
             sendSessionReplicationMessage(request, clusterManager);
             if (isCrossContext) {
                 sendCrossContextSession();
             }
-        } catch (Exception x) {
+        } catch (Exception e) {
             // FIXME we have a lot of sends, but the trouble with one node stops the correct replication to other nodes!
-            log.error(sm.getString("ReplicationValve.send.failure"), x);
+            log.error(sm.getString("ReplicationValve.send.failure"), e);
         } finally {
             if (doStatistics()) {
-                updateStats(totalstart, start);
+                updateStats(totalstart, start, isAsync);
             }
         }
     }
@@ -413,10 +392,10 @@ public class ReplicationValve extends ValveBase implements ClusterValve {
      */
     protected void sendCrossContextSession() {
         List<DeltaSession> sessions = crossContextSessions.get();
-        if (sessions != null && sessions.size() > 0) {
+        if (sessions != null && !sessions.isEmpty()) {
             for (DeltaSession session : sessions) {
-                if (log.isDebugEnabled()) {
-                    log.debug(sm.getString("ReplicationValve.crossContext.sendDelta",
+                if (log.isTraceEnabled()) {
+                    log.trace(sm.getString("ReplicationValve.crossContext.sendDelta",
                             session.getManager().getContext().getName()));
                 }
                 sendMessage(session, (ClusterManager) session.getManager());
@@ -441,15 +420,10 @@ public class ReplicationValve extends ValveBase implements ClusterValve {
         }
         if (isCrossContext) {
             List<DeltaSession> sessions = crossContextSessions.get();
-            if (sessions != null && sessions.size() > 0) {
-                Iterator<DeltaSession> iter = sessions.iterator();
-                for (; iter.hasNext();) {
-                    Session session = iter.next();
+            if (sessions != null) {
+                for (DeltaSession session : sessions) {
                     resetDeltaRequest(session);
-                    if (session instanceof DeltaSession) {
-                        ((DeltaSession) contextSession).setPrimarySession(true);
-                    }
-
+                    session.setPrimarySession(true);
                 }
             }
         }
@@ -529,13 +503,11 @@ public class ReplicationValve extends ValveBase implements ClusterValve {
      */
     protected void sendInvalidSessions(ClusterManager manager) {
         String[] invalidIds = manager.getInvalidatedSessions();
-        if (invalidIds.length > 0) {
-            for (String invalidId : invalidIds) {
-                try {
-                    send(manager, invalidId);
-                } catch (Exception x) {
-                    log.error(sm.getString("ReplicationValve.send.invalid.failure", invalidId), x);
-                }
+        for (String invalidId : invalidIds) {
+            try {
+                send(manager, invalidId);
+            } catch (Exception e) {
+                log.error(sm.getString("ReplicationValve.send.invalid.failure", invalidId), e);
             }
         }
     }
@@ -557,23 +529,25 @@ public class ReplicationValve extends ValveBase implements ClusterValve {
      *
      * @param requestTime Request time
      * @param clusterTime Cluster time
+     * @param isAsync     if the request was in async mode
      */
-    protected void updateStats(long requestTime, long clusterTime) {
-        // TODO: Async requests may trigger multiple replication requests. How,
-        // if at all, should the stats handle this?
+    protected void updateStats(long requestTime, long clusterTime, boolean isAsync) {
         long currentTime = System.currentTimeMillis();
         lastSendTime.set(currentTime);
         totalSendTime.add(currentTime - clusterTime);
         totalRequestTime.add(currentTime - requestTime);
-        nrOfRequests.increment();
-        if (log.isInfoEnabled()) {
-            if ((nrOfRequests.longValue() % 100) == 0) {
-                log.info(sm.getString("ReplicationValve.stats",
-                        new Object[] { Long.valueOf(totalRequestTime.longValue() / nrOfRequests.longValue()),
-                                Long.valueOf(totalSendTime.longValue() / nrOfRequests.longValue()), Long.valueOf(nrOfRequests.longValue()),
-                                Long.valueOf(nrOfSendRequests.longValue()), Long.valueOf(nrOfCrossContextSendRequests.longValue()),
-                                Long.valueOf(nrOfFilterRequests.longValue()), Long.valueOf(totalRequestTime.longValue()),
-                                Long.valueOf(totalSendTime.longValue()) }));
+        if (!isAsync) {
+            nrOfRequests.increment();
+            if (log.isDebugEnabled()) {
+                if ((nrOfRequests.longValue() % 100) == 0) {
+                    log.debug(sm.getString("ReplicationValve.stats",
+                            Long.valueOf(totalRequestTime.longValue() / nrOfRequests.longValue()),
+                            Long.valueOf(totalSendTime.longValue() / nrOfRequests.longValue()),
+                            Long.valueOf(nrOfRequests.longValue()), Long.valueOf(nrOfSendRequests.longValue()),
+                            Long.valueOf(nrOfCrossContextSendRequests.longValue()),
+                            Long.valueOf(nrOfFilterRequests.longValue()), Long.valueOf(totalRequestTime.longValue()),
+                            Long.valueOf(totalSendTime.longValue())));
+                }
             }
         }
     }
@@ -588,7 +562,7 @@ public class ReplicationValve extends ValveBase implements ClusterValve {
      */
     protected void createPrimaryIndicator(Request request) throws IOException {
         String id = request.getRequestedSessionId();
-        if ((id != null) && (id.length() > 0)) {
+        if ((id != null) && (!id.isEmpty())) {
             Manager manager = request.getContext().getManager();
             Session session = manager.findSession(id);
             if (session instanceof ClusterSession) {
