@@ -1113,7 +1113,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
                         stream = url.openStream();
                     }
                 }
-            } catch (IOException e) {
+            } catch (IOException ignore) {
                 // Ignore
             }
             if (stream != null) {
@@ -1370,9 +1370,8 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
         // It is not permitted to load resources once the web application has
         // been stopped.
         if (!state.isAvailable()) {
-            String msg = sm.getString("webappClassLoader.stopped", resource);
-            IllegalStateException ise = new IllegalStateException(msg);
-            log.info(msg, ise);
+            IllegalStateException ise = new IllegalStateException(sm.getString("webappClassLoader.stopped", resource));
+            log.info(ise.getMessage(), ise);
             throw ise;
         }
     }
@@ -2327,7 +2326,7 @@ public abstract class WebappClassLoaderBase extends URLClassLoader
                 if (pkg != null) {
                     boolean sealCheck = true;
                     if (pkg.isSealed()) {
-                        if(codeBase != null) {
+                        if (codeBase != null) {
                             sealCheck = pkg.isSealed(codeBase);
                         } else {
                             sealCheck = false;
