@@ -32,15 +32,16 @@ import javax.servlet.ServletContext;
 
 import org.apache.catalina.Context;
 import org.apache.catalina.Globals;
-import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Loader;
 import org.apache.catalina.util.LifecycleMBeanBase;
+import org.apache.catalina.util.ToStringUtil;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.buf.UDecoder;
+import org.apache.tomcat.util.compat.JreCompat;
 import org.apache.tomcat.util.modeler.Registry;
 import org.apache.tomcat.util.res.StringManager;
 
@@ -58,6 +59,7 @@ import org.apache.tomcat.util.res.StringManager;
  */
 public class WebappLoader extends LifecycleMBeanBase implements Loader, PropertyChangeListener {
 
+    private static final Log log = LogFactory.getLog(WebappLoader.class);
 
     // ----------------------------------------------------------- Constructors
 
@@ -127,7 +129,7 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
     /**
      * The string manager for this package.
      */
-    protected static final StringManager sm = StringManager.getManager(Constants.Package);
+    protected static final StringManager sm = StringManager.getManager(WebappLoader.class);
 
 
     /**
@@ -144,9 +146,6 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
 
     // ------------------------------------------------------------- Properties
 
-    /**
-     * Return the Java class loader to be used by this Container.
-     */
     @Override
     public ClassLoader getClassLoader() {
         return classLoader;
@@ -188,20 +187,12 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
     }
 
 
-    /**
-     * Return the "follow standard delegation model" flag used to configure our ClassLoader.
-     */
     @Override
     public boolean getDelegate() {
         return this.delegate;
     }
 
 
-    /**
-     * Set the "follow standard delegation model" flag used to configure our ClassLoader.
-     *
-     * @param delegate The new flag
-     */
     @Override
     public void setDelegate(boolean delegate) {
         boolean oldDelegate = this.delegate;
@@ -227,6 +218,17 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
         this.loaderClass = loaderClass;
     }
 
+    /**
+     * Set the ClassLoader instance, without relying on reflection This method will also invoke
+     * {@link #setLoaderClass(String)} with {@code loaderInstance.getClass().getName()} as an argument
+     *
+     * @param loaderInstance The new ClassLoader instance to use
+     */
+    public void setLoaderInstance(WebappClassLoaderBase loaderInstance) {
+        this.classLoader = loaderInstance;
+        setLoaderClass(loaderInstance.getClass().getName());
+    }
+
 
     /**
      * Return the reloadable flag for this Loader.
@@ -253,11 +255,6 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
 
     // --------------------------------------------------------- Public Methods
 
-    /**
-     * Add a property change listener to this component.
-     *
-     * @param listener The listener to add
-     */
     @Override
     public void addPropertyChangeListener(PropertyChangeListener listener) {
 
@@ -266,10 +263,6 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
     }
 
 
-    /**
-     * Execute a periodic task, such as reloading, etc. This method will be invoked inside the classloading context of
-     * this container. Unexpected throwables will be caught and logged.
-     */
     @Override
     public void backgroundProcess() {
         if (reloadable && modified()) {
@@ -301,7 +294,7 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
     }
 
     public String getLoaderRepositoriesString() {
-        String repositories[] = getLoaderRepositories();
+        String[] repositories = getLoaderRepositories();
         StringBuilder sb = new StringBuilder();
         for (String repository : repositories) {
             sb.append(repository).append(':');
@@ -320,38 +313,21 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
     }
 
 
-    /**
-     * Has the internal repository associated with this Loader been modified, such that the loaded classes should be
-     * reloaded?
-     */
     @Override
     public boolean modified() {
-        return classLoader != null ? classLoader.modified() : false;
+        return classLoader != null && classLoader.modified();
     }
 
 
-    /**
-     * Remove a property change listener from this component.
-     *
-     * @param listener The listener to remove
-     */
     @Override
     public void removePropertyChangeListener(PropertyChangeListener listener) {
         support.removePropertyChangeListener(listener);
     }
 
 
-    /**
-     * Return a String representation of this component.
-     */
     @Override
     public String toString() {
-        StringBuilder sb = new StringBuilder("WebappLoader[");
-        if (context != null) {
-            sb.append(context.getName());
-        }
-        sb.append("]");
-        return sb.toString();
+        return ToStringUtil.toString(this, context);
     }
 
 
@@ -370,7 +346,7 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
         }
 
         if (context.getResources() == null) {
-            log.info("No resources for " + context);
+            log.info(sm.getString("webappLoader.noResources", context));
             setState(LifecycleState.STARTING);
             return;
         }
@@ -387,7 +363,7 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
 
             setPermissions();
 
-            ((Lifecycle) classLoader).start();
+            classLoader.start();
 
             String contextName = context.getName();
             if (!contextName.startsWith("/")) {
@@ -396,13 +372,12 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
             ObjectName cloname =
                     new ObjectName(context.getDomain() + ":type=" + classLoader.getClass().getSimpleName() + ",host=" +
                             context.getParent().getName() + ",context=" + contextName);
-            Registry.getRegistry(null, null).registerComponent(classLoader, cloname, null);
+            Registry.getRegistry(null).registerComponent(classLoader, cloname, null);
 
         } catch (Throwable t) {
-            t = ExceptionUtils.unwrapInvocationTargetException(t);
-            ExceptionUtils.handleThrowable(t);
-            log.error("LifecycleException ", t);
-            throw new LifecycleException("start: ", t);
+            Throwable throwable = ExceptionUtils.unwrapInvocationTargetException(t);
+            ExceptionUtils.handleThrowable(throwable);
+            throw new LifecycleException(sm.getString("webappLoader.startError"), throwable);
         }
 
         setState(LifecycleState.STARTING);
@@ -446,9 +421,9 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
                 ObjectName cloname =
                         new ObjectName(context.getDomain() + ":type=" + classLoader.getClass().getSimpleName() +
                                 ",host=" + context.getParent().getName() + ",context=" + contextName);
-                Registry.getRegistry(null, null).unregisterComponent(cloname);
+                Registry.getRegistry(null).unregisterComponent(cloname);
             } catch (Exception e) {
-                log.warn("LifecycleException ", e);
+                log.warn(sm.getString("webappLoader.stopError"), e);
             }
         }
 
@@ -491,20 +466,27 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
      */
     private WebappClassLoaderBase createClassLoader() throws Exception {
 
-        Class<?> clazz = Class.forName(loaderClass);
-        WebappClassLoaderBase classLoader = null;
+        if (classLoader != null) {
+            return classLoader;
+        }
 
         if (parentClassLoader == null) {
             parentClassLoader = context.getParentClassLoader();
         } else {
             context.setParentClassLoader(parentClassLoader);
         }
+
+        if (ParallelWebappClassLoader.class.getName().equals(loaderClass)) {
+            return new ParallelWebappClassLoader(parentClassLoader);
+        }
+
+        Class<?> clazz = Class.forName(loaderClass);
+
         Class<?>[] argTypes = { ClassLoader.class };
         Object[] args = { parentClassLoader };
         Constructor<?> constr = clazz.getConstructor(argTypes);
-        classLoader = (WebappClassLoaderBase) constr.newInstance(args);
 
-        return classLoader;
+        return (WebappClassLoaderBase) constr.newInstance(args);
     }
 
 
@@ -589,13 +571,13 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
 
     private boolean buildClassPath(StringBuilder classpath, ClassLoader loader) {
         if (loader instanceof URLClassLoader) {
-            URL repositories[] = ((URLClassLoader) loader).getURLs();
+            URL[] repositories = ((URLClassLoader) loader).getURLs();
             for (URL url : repositories) {
                 String repository = url.toString();
                 if (repository.startsWith("file://")) {
-                    repository = UDecoder.URLDecode(repository.substring(7), StandardCharsets.ISO_8859_1);
+                    repository = UDecoder.URLDecode(repository.substring(7), StandardCharsets.UTF_8);
                 } else if (repository.startsWith("file:")) {
-                    repository = UDecoder.URLDecode(repository.substring(5), StandardCharsets.ISO_8859_1);
+                    repository = UDecoder.URLDecode(repository.substring(5), StandardCharsets.UTF_8);
                 } else {
                     continue;
                 }
@@ -611,7 +593,7 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
             // Java 9 onwards. The internal class loaders no longer extend
             // URLCLassLoader
             String cp = System.getProperty("java.class.path");
-            if (cp != null && cp.length() > 0) {
+            if (cp != null && !cp.isEmpty()) {
                 if (classpath.length() > 0) {
                     classpath.append(File.pathSeparator);
                 }
@@ -619,16 +601,15 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader, Property
             }
             return false;
         } else {
-            log.info("Unknown loader " + loader + " " + loader.getClass());
+            // Ignore Graal "unknown" classloader
+            if (!JreCompat.isGraalAvailable()) {
+                log.info(sm.getString("webappLoader.unknownClassLoader", loader, loader.getClass()));
+            }
             return false;
         }
         return true;
     }
 
-
-    private static final Log log = LogFactory.getLog(WebappLoader.class);
-
-
     @Override
     protected String getDomainInternal() {
         return context.getDomain();
