@@ -5,17 +5,15 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.realm;
 
 
@@ -29,29 +27,26 @@ import java.util.ArrayList;
 import java.util.Properties;
 
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.util.StringManager;
+import org.apache.tomcat.util.ExceptionUtils;
 
 
 /**
-*
-* Implmentation of <b>Realm</b> that works with any JDBC supported database.
-* See the JDBCRealm.howto for more details on how to set up the database and
-* for configuration options.
-*
-* <p>For a <b>Realm</b> implementation that supports connection pooling and
-* doesn't require synchronisation of <code>authenticate()</code>,
-* <code>getPassword()</code>, <code>roles()</code> and
-* <code>getPrincipal()</code> or the ugly connection logic use the
-* <code>DataSourceRealm</code>.</p>
-*
-* @author Craig R. McClanahan
-* @author Carson McDonald
-* @author Ignacio Ortega
-*
-*/
-
-public class JDBCRealm
-    extends RealmBase {
+ * Implementation of <b>Realm</b> that works with any JDBC supported database. See the JDBCRealm.howto for more details
+ * on how to set up the database and for configuration options.
+ * <p>
+ * For a <b>Realm</b> implementation that supports connection pooling and doesn't require synchronisation of
+ * <code>authenticate()</code>, <code>getPassword()</code>, <code>roles()</code> and <code>getPrincipal()</code> or the
+ * ugly connection logic use the <code>DataSourceRealm</code>.
+ * </p>
+ *
+ * @author Craig R. McClanahan
+ * @author Carson McDonald
+ * @author Ignacio Ortega
+ *
+ * @deprecated Will be removed in Tomcat 10 onwards. Use the DataSourceRealm instead.
+ */
+@Deprecated
+public class JDBCRealm extends RealmBase {
 
 
     // ----------------------------------------------------- Instance Variables
@@ -93,19 +88,6 @@ public class JDBCRealm
     protected String driverName = null;
 
 
-    /**
-     * Descriptive information about this Realm implementation.
-     */
-    protected static final String info =
-        "org.apache.catalina.realm.JDBCRealm/1.0";
-
-
-    /**
-     * Descriptive information about this Realm implementation.
-     */
-    protected static final String name = "JDBCRealm";
-
-
     /**
      * The PreparedStatement to use for authenticating users.
      */
@@ -113,8 +95,7 @@ public class JDBCRealm
 
 
     /**
-     * The PreparedStatement to use for identifying the roles for
-     * a specified user.
+     * The PreparedStatement to use for identifying the roles for a specified user.
      */
     protected PreparedStatement preparedRoles = null;
 
@@ -126,14 +107,7 @@ public class JDBCRealm
 
 
     /**
-     * The string manager for this package.
-     */
-    protected static final StringManager sm =
-        StringManager.getManager(Constants.Package);
-
-
-    /**
-     * The column in the user table that holds the user's credintials
+     * The column in the user table that holds the user's credentials
      */
     protected String userCredCol = null;
 
@@ -159,8 +133,7 @@ public class JDBCRealm
     // ------------------------------------------------------------- Properties
 
     /**
-     * Return the username to use to connect to the database.
-     *
+     * @return the username to use to connect to the database.
      */
     public String getConnectionName() {
         return connectionName;
@@ -176,8 +149,7 @@ public class JDBCRealm
     }
 
     /**
-     * Return the password to use to connect to the database.
-     *
+     * @return the password to use to connect to the database.
      */
     public String getConnectionPassword() {
         return connectionPassword;
@@ -193,8 +165,7 @@ public class JDBCRealm
     }
 
     /**
-     * Return the URL to use to connect to the database.
-     *
+     * @return the URL to use to connect to the database.
      */
     public String getConnectionURL() {
         return connectionURL;
@@ -205,13 +176,12 @@ public class JDBCRealm
      *
      * @param connectionURL The new connection URL
      */
-    public void setConnectionURL( String connectionURL ) {
-      this.connectionURL = connectionURL;
+    public void setConnectionURL(String connectionURL) {
+        this.connectionURL = connectionURL;
     }
 
     /**
-     * Return the JDBC driver that will be used.
-     *
+     * @return the JDBC driver that will be used.
      */
     public String getDriverName() {
         return driverName;
@@ -222,13 +192,12 @@ public class JDBCRealm
      *
      * @param driverName The driver name
      */
-    public void setDriverName( String driverName ) {
-      this.driverName = driverName;
+    public void setDriverName(String driverName) {
+        this.driverName = driverName;
     }
 
     /**
-     * Return the column in the user role table that names a role.
-     *
+     * @return the column in the user role table that names a role.
      */
     public String getRoleNameCol() {
         return roleNameCol;
@@ -239,13 +208,12 @@ public class JDBCRealm
      *
      * @param roleNameCol The column name
      */
-    public void setRoleNameCol( String roleNameCol ) {
+    public void setRoleNameCol(String roleNameCol) {
         this.roleNameCol = roleNameCol;
     }
 
     /**
-     * Return the column in the user table that holds the user's credentials.
-     *
+     * @return the column in the user table that holds the user's credentials.
      */
     public String getUserCredCol() {
         return userCredCol;
@@ -256,13 +224,12 @@ public class JDBCRealm
      *
      * @param userCredCol The column name
      */
-    public void setUserCredCol( String userCredCol ) {
-       this.userCredCol = userCredCol;
+    public void setUserCredCol(String userCredCol) {
+        this.userCredCol = userCredCol;
     }
 
     /**
-     * Return the column in the user table that holds the user's name.
-     *
+     * @return the column in the user table that holds the user's name.
      */
     public String getUserNameCol() {
         return userNameCol;
@@ -273,13 +240,12 @@ public class JDBCRealm
      *
      * @param userNameCol The column name
      */
-    public void setUserNameCol( String userNameCol ) {
-       this.userNameCol = userNameCol;
+    public void setUserNameCol(String userNameCol) {
+        this.userNameCol = userNameCol;
     }
 
     /**
-     * Return the table that holds the relation between user's and roles.
-     *
+     * @return the table that holds the relation between user's and roles.
      */
     public String getUserRoleTable() {
         return userRoleTable;
@@ -290,13 +256,12 @@ public class JDBCRealm
      *
      * @param userRoleTable The table name
      */
-    public void setUserRoleTable( String userRoleTable ) {
+    public void setUserRoleTable(String userRoleTable) {
         this.userRoleTable = userRoleTable;
     }
 
     /**
-     * Return the table that holds user data..
-     *
+     * @return the table that holds user data..
      */
     public String getUserTable() {
         return userTable;
@@ -307,64 +272,48 @@ public class JDBCRealm
      *
      * @param userTable The table name
      */
-    public void setUserTable( String userTable ) {
-      this.userTable = userTable;
+    public void setUserTable(String userTable) {
+        this.userTable = userTable;
     }
 
-    /**
-     * Return descriptive information about this Realm implementation and
-     * the corresponding version number, in the format
-     * <code>&lt;description&gt;/&lt;version&gt;</code>.
-     */
-    @Override
-    public String getInfo() {
-
-        return info;
-
-    }
 
     // --------------------------------------------------------- Public Methods
 
-
     /**
-     * Return the Principal associated with the specified username and
-     * credentials, if there is one; otherwise return <code>null</code>.
+     * Return the Principal associated with the specified username and credentials, if there is one; otherwise return
+     * <code>null</code>. If there are any errors with the JDBC connection, executing the query or anything we return
+     * null (don't authenticate). This event is also logged, and the connection will be closed so that a subsequent
+     * request will automatically re-open it.
      *
-     * If there are any errors with the JDBC connection, executing
-     * the query or anything we return null (don't authenticate). This
-     * event is also logged, and the connection will be closed so that
-     * a subsequent request will automatically re-open it.
+     * @param username    Username of the Principal to look up
+     * @param credentials Password or other credentials to use in authenticating this username
      *
-     *
-     * @param username Username of the Principal to look up
-     * @param credentials Password or other credentials to use in
-     *  authenticating this username
+     * @return the associated principal, or <code>null</code> if there is none.
      */
     @Override
     public synchronized Principal authenticate(String username, String credentials) {
 
-        // Number of tries is the numebr of attempts to connect to the database
+        // Number of tries is the number of attempts to connect to the database
         // during this login attempt (if we need to open the database)
-        // This needs rewritten wuth better pooling support, the existing code
+        // This needs rewritten with better pooling support, the existing code
         // needs signature changes since the Prepared statements needs cached
         // with the connections.
-        // The code below will try twice if there is a SQLException so the
+        // The code below will try twice if there is an SQLException so the
         // connection may try to be opened again. On normal conditions (including
         // invalid login - the above is only used once.
         int numberOfTries = 2;
-        while (numberOfTries>0) {
+        while (numberOfTries > 0) {
             try {
 
                 // Ensure that we have an open database connection
                 open();
 
                 // Acquire a Principal object for this user
-                Principal principal = authenticate(dbConnection,
-                                                   username, credentials);
+                Principal principal = authenticate(dbConnection, username, credentials);
 
 
                 // Return the Principal (if any)
-                return (principal);
+                return principal;
 
             } catch (SQLException e) {
 
@@ -372,8 +321,9 @@ public class JDBCRealm
                 containerLog.error(sm.getString("jdbcRealm.exception"), e);
 
                 // Close the connection so that it gets reopened next time
-                if (dbConnection != null)
+                if (dbConnection != null) {
                     close(dbConnection);
+                }
 
             }
 
@@ -393,52 +343,63 @@ public class JDBCRealm
 
 
     /**
-     * Return the Principal associated with the specified username and
-     * credentials, if there is one; otherwise return <code>null</code>.
+     * Attempt to authenticate the user with the provided credentials.
      *
      * @param dbConnection The database connection to be used
-     * @param username Username of the Principal to look up
-     * @param credentials Password or other credentials to use in
-     *  authenticating this username
+     * @param username     Username of the Principal to look up
+     * @param credentials  Password or other credentials to use in authenticating this username
+     *
+     * @return Return the Principal associated with the specified username and credentials, if there is one; otherwise
+     *             return <code>null</code>.
      */
-    public synchronized Principal authenticate(Connection dbConnection,
-                                               String username,
-                                               String credentials) {
-
+    public synchronized Principal authenticate(Connection dbConnection, String username, String credentials) {
         // No user or no credentials
         // Can't possibly authenticate, don't bother the database then
         if (username == null || credentials == null) {
+            if (containerLog.isTraceEnabled()) {
+                containerLog.trace(sm.getString("jdbcRealm.authenticateFailure", username));
+            }
             return null;
         }
 
         // Look up the user's credentials
         String dbCredentials = getPassword(username);
 
-        // Validate the user's credentials
-        boolean validated = false;
-        if (hasMessageDigest()) {
-            // Hex hashes should be compared case-insensitive
-            validated = (digest(credentials).equalsIgnoreCase(dbCredentials));
-        } else {
-            validated = (digest(credentials).equals(dbCredentials));
+        if (dbCredentials == null) {
+            // User was not found in the database.
+            // Waste a bit of time as not to reveal that the user does not exist.
+            getCredentialHandler().mutate(credentials);
+
+            if (containerLog.isTraceEnabled()) {
+                containerLog.trace(sm.getString("jdbcRealm.authenticateFailure", username));
+            }
+            return null;
         }
 
+        // Validate the user's credentials
+        boolean validated = getCredentialHandler().matches(credentials, dbCredentials);
+
         if (validated) {
-            if (containerLog.isTraceEnabled())
-                containerLog.trace(sm.getString("jdbcRealm.authenticateSuccess",
-                                                username));
+            if (containerLog.isTraceEnabled()) {
+                containerLog.trace(sm.getString("jdbcRealm.authenticateSuccess", username));
+            }
         } else {
-            if (containerLog.isTraceEnabled())
-                containerLog.trace(sm.getString("jdbcRealm.authenticateFailure",
-                                                username));
-            return (null);
+            if (containerLog.isTraceEnabled()) {
+                containerLog.trace(sm.getString("jdbcRealm.authenticateFailure", username));
+            }
+            return null;
         }
 
         ArrayList<String> roles = getRoles(username);
-        
-        // Create and return a suitable Principal for this user
-        return (new GenericPrincipal(this, username, credentials, roles));
 
+        // Create and return a suitable Principal for this user
+        return new GenericPrincipal(username, credentials, roles);
+    }
+
+
+    @Override
+    public boolean isAvailable() {
+        return (dbConnection != null);
     }
 
 
@@ -450,14 +411,15 @@ public class JDBCRealm
     protected void close(Connection dbConnection) {
 
         // Do nothing if the database connection is already closed
-        if (dbConnection == null)
+        if (dbConnection == null) {
             return;
+        }
 
         // Close our prepared statements (if any)
         try {
             preparedCredentials.close();
         } catch (Throwable f) {
-            ;
+            ExceptionUtils.handleThrowable(f);
         }
         this.preparedCredentials = null;
 
@@ -465,7 +427,7 @@ public class JDBCRealm
         try {
             preparedRoles.close();
         } catch (Throwable f) {
-            ;
+            ExceptionUtils.handleThrowable(f);
         }
         this.preparedRoles = null;
 
@@ -476,27 +438,27 @@ public class JDBCRealm
         } catch (SQLException e) {
             containerLog.warn(sm.getString("jdbcRealm.close"), e); // Just log it here
         } finally {
-           this.dbConnection = null;
+            this.dbConnection = null;
         }
 
     }
 
 
     /**
-     * Return a PreparedStatement configured to perform the SELECT required
-     * to retrieve user credentials for the specified username.
+     * Return a PreparedStatement configured to perform the SELECT required to retrieve user credentials for the
+     * specified username.
      *
      * @param dbConnection The database connection to be used
-     * @param username Username for which credentials should be retrieved
+     * @param username     Username for which credentials should be retrieved
+     *
+     * @return the prepared statement
      *
      * @exception SQLException if a database error occurs
      */
-    protected PreparedStatement credentials(Connection dbConnection,
-                                            String username)
-        throws SQLException {
+    protected PreparedStatement credentials(Connection dbConnection, String username) throws SQLException {
 
         if (preparedCredentials == null) {
-            StringBuffer sb = new StringBuffer("SELECT ");
+            StringBuilder sb = new StringBuilder("SELECT ");
             sb.append(userCredCol);
             sb.append(" FROM ");
             sb.append(userTable);
@@ -504,269 +466,226 @@ public class JDBCRealm
             sb.append(userNameCol);
             sb.append(" = ?");
 
-            if(containerLog.isDebugEnabled()) {
+            if (containerLog.isDebugEnabled()) {
                 containerLog.debug("credentials query: " + sb.toString());
             }
 
-            preparedCredentials =
-                dbConnection.prepareStatement(sb.toString());
+            preparedCredentials = dbConnection.prepareStatement(sb.toString());
         }
 
         if (username == null) {
-            preparedCredentials.setNull(1,java.sql.Types.VARCHAR);
+            preparedCredentials.setNull(1, java.sql.Types.VARCHAR);
         } else {
             preparedCredentials.setString(1, username);
         }
 
-        return (preparedCredentials);
+        return preparedCredentials;
     }
 
 
     /**
-     * Return a short name for this Realm implementation.
-     */
-    protected String getName() {
-
-        return (name);
-
-    }
-
-
-    /**
-     * Return the password associated with the given principal's user name.
+     * Get the password for the specified user.
+     *
+     * @param username The user name
+     *
+     * @return the password associated with the given principal's user name.
      */
+    @Override
     protected synchronized String getPassword(String username) {
 
         // Look up the user's credentials
         String dbCredentials = null;
-        PreparedStatement stmt = null;
-        ResultSet rs = null;
 
-        // Number of tries is the numebr of attempts to connect to the database
+        // Number of tries is the number of attempts to connect to the database
         // during this login attempt (if we need to open the database)
-        // This needs rewritten wuth better pooling support, the existing code
+        // This needs rewritten with better pooling support, the existing code
         // needs signature changes since the Prepared statements needs cached
         // with the connections.
-        // The code below will try twice if there is a SQLException so the
+        // The code below will try twice if there is an SQLException so the
         // connection may try to be opened again. On normal conditions (including
         // invalid login - the above is only used once.
         int numberOfTries = 2;
-        while (numberOfTries>0) {
+        while (numberOfTries > 0) {
             try {
-                
                 // Ensure that we have an open database connection
                 open();
-                
-                try {
-                    stmt = credentials(dbConnection, username);
-                    rs = stmt.executeQuery();
-                    
+
+                PreparedStatement stmt = credentials(dbConnection, username);
+                try (ResultSet rs = stmt.executeQuery()) {
                     if (rs.next()) {
                         dbCredentials = rs.getString(1);
                     }
-                    rs.close();
-                    rs = null;
-                    if (dbCredentials == null) {
-                        return (null);
-                    }
-                    
-                    dbCredentials = dbCredentials.trim();
-                    return dbCredentials;
-                    
-                } finally {
-                    if (rs!=null) {
-                        try {
-                            rs.close();
-                        } catch(SQLException e) {
-                            containerLog.warn(sm.getString("jdbcRealm.abnormalCloseResultSet"));
-                        }
-                    }
+
                     dbConnection.commit();
+
+                    if (dbCredentials != null) {
+                        dbCredentials = dbCredentials.trim();
+                    }
+
+                    return dbCredentials;
                 }
-                
             } catch (SQLException e) {
-                
                 // Log the problem for posterity
                 containerLog.error(sm.getString("jdbcRealm.exception"), e);
-                
-                // Close the connection so that it gets reopened next time
-                if (dbConnection != null)
-                    close(dbConnection);
-                
             }
-            
+
+            // Close the connection so that it gets reopened next time
+            if (dbConnection != null) {
+                close(dbConnection);
+            }
+
             numberOfTries--;
         }
-        
-        return (null);
+
+        return null;
     }
 
-
     /**
-     * Return the Principal associated with the given user name.
+     * Get the principal associated with the specified user.
+     *
+     * @param username The user name
+     *
+     * @return the Principal associated with the given user name.
      */
+    @Override
     protected synchronized Principal getPrincipal(String username) {
 
-        return (new GenericPrincipal(this,
-                                     username,
-                                     getPassword(username),
-                                     getRoles(username)));
+        return new GenericPrincipal(username, getPassword(username), getRoles(username));
 
     }
 
 
     /**
-     * Return the roles associated with the gven user name.
+     * Return the roles associated with the given user name.
+     *
+     * @param username The user name
+     *
+     * @return an array list of the role names
      */
     protected ArrayList<String> getRoles(String username) {
-        
+
         if (allRolesMode != AllRolesMode.STRICT_MODE && !isRoleStoreDefined()) {
             // Using an authentication only configuration and no role store has
             // been defined so don't spend cycles looking
             return null;
         }
 
-        PreparedStatement stmt = null;
-        ResultSet rs = null;
-
-        // Number of tries is the numebr of attempts to connect to the database
+        // Number of tries is the number of attempts to connect to the database
         // during this login attempt (if we need to open the database)
         // This needs rewritten wuth better pooling support, the existing code
         // needs signature changes since the Prepared statements needs cached
         // with the connections.
-        // The code below will try twice if there is a SQLException so the
+        // The code below will try twice if there is an SQLException so the
         // connection may try to be opened again. On normal conditions (including
         // invalid login - the above is only used once.
         int numberOfTries = 2;
-        while (numberOfTries>0) {
+        while (numberOfTries > 0) {
             try {
-                
                 // Ensure that we have an open database connection
                 open();
-                
-                try {
+
+                PreparedStatement stmt = roles(dbConnection, username);
+                try (ResultSet rs = stmt.executeQuery()) {
                     // Accumulate the user's roles
-                    ArrayList<String> roleList = new ArrayList<String>();
-                    stmt = roles(dbConnection, username);
-                    rs = stmt.executeQuery();
+                    ArrayList<String> roleList = new ArrayList<>();
+
                     while (rs.next()) {
                         String role = rs.getString(1);
-                        if (null!=role) {
+                        if (null != role) {
                             roleList.add(role.trim());
                         }
                     }
-                    rs.close();
-                    rs = null;
-                    
-                    return (roleList);
-                    
+
+                    return roleList;
                 } finally {
-                    if (rs!=null) {
-                        try {
-                            rs.close();
-                        } catch(SQLException e) {
-                            containerLog.warn(sm.getString("jdbcRealm.abnormalCloseResultSet"));
-                        }
-                    }
                     dbConnection.commit();
                 }
-                
             } catch (SQLException e) {
-                
                 // Log the problem for posterity
                 containerLog.error(sm.getString("jdbcRealm.exception"), e);
-                
+
                 // Close the connection so that it gets reopened next time
-                if (dbConnection != null)
+                if (dbConnection != null) {
                     close(dbConnection);
-                
+                }
             }
-            
+
             numberOfTries--;
         }
-        
+
         return null;
     }
-    
-    
+
+
     /**
-     * Open (if necessary) and return a database connection for use by
-     * this Realm.
+     * Open (if necessary) and return a database connection for use by this Realm.
+     *
+     * @return the opened connection
      *
      * @exception SQLException if a database error occurs
      */
     protected Connection open() throws SQLException {
 
         // Do nothing if there is a database connection already open
-        if (dbConnection != null)
-            return (dbConnection);
+        if (dbConnection != null) {
+            return dbConnection;
+        }
 
         // Instantiate our database driver if necessary
         if (driver == null) {
             try {
-                Class clazz = Class.forName(driverName);
-                driver = (Driver) clazz.newInstance();
+                Class<?> clazz = Class.forName(driverName);
+                driver = (Driver) clazz.getConstructor().newInstance();
             } catch (Throwable e) {
-                throw new SQLException(e.getMessage());
+                ExceptionUtils.handleThrowable(e);
+                throw new SQLException(e.getMessage(), e);
             }
         }
 
         // Open a new connection
         Properties props = new Properties();
-        if (connectionName != null)
+        if (connectionName != null) {
             props.put("user", connectionName);
-        if (connectionPassword != null)
+        }
+        if (connectionPassword != null) {
             props.put("password", connectionPassword);
+        }
         dbConnection = driver.connect(connectionURL, props);
         if (dbConnection == null) {
-            throw new SQLException(sm.getString(
-                    "jdbcRealm.open.invalidurl",driverName, connectionURL));
+            throw new SQLException(sm.getString("jdbcRealm.open.invalidurl", driverName, connectionURL));
         }
         dbConnection.setAutoCommit(false);
-        return (dbConnection);
+        return dbConnection;
 
     }
 
 
     /**
-     * Release our use of this connection so that it can be recycled.
-     *
-     * @param dbConnection The connection to be released
-     */
-    protected void release(Connection dbConnection) {
-
-        ; // NO-OP since we are not pooling anything
-
-    }
-
-
-    /**
-     * Return a PreparedStatement configured to perform the SELECT required
-     * to retrieve user roles for the specified username.
+     * Return a PreparedStatement configured to perform the SELECT required to retrieve user roles for the specified
+     * username.
      *
      * @param dbConnection The database connection to be used
-     * @param username Username for which roles should be retrieved
+     * @param username     Username for which roles should be retrieved
+     *
+     * @return the prepared statement
      *
      * @exception SQLException if a database error occurs
      */
-    protected synchronized PreparedStatement roles(Connection dbConnection,
-            String username)
-        throws SQLException {
+    protected synchronized PreparedStatement roles(Connection dbConnection, String username) throws SQLException {
 
         if (preparedRoles == null) {
-            StringBuffer sb = new StringBuffer("SELECT ");
+            StringBuilder sb = new StringBuilder("SELECT ");
             sb.append(roleNameCol);
             sb.append(" FROM ");
             sb.append(userRoleTable);
             sb.append(" WHERE ");
             sb.append(userNameCol);
             sb.append(" = ?");
-            preparedRoles =
-                dbConnection.prepareStatement(sb.toString());
+            preparedRoles = dbConnection.prepareStatement(sb.toString());
         }
 
         preparedRoles.setString(1, username);
-        return (preparedRoles);
+        return preparedRoles;
 
     }
 
@@ -779,16 +698,14 @@ public class JDBCRealm
     // ------------------------------------------------------ Lifecycle Methods
 
     /**
+     * Prepare for the beginning of active use of the public methods of this component and implement the requirements of
+     * {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
      *
-     * Prepare for active use of the public methods of this Component.
-     *
-     * @exception LifecycleException if this component detects a fatal error
-     *  that prevents it from being started
+     * @exception LifecycleException if this component detects a fatal error that prevents this component from being
+     *                                   used
      */
-    public void start() throws LifecycleException {
-
-        // Perform normal superclass initialization
-        super.start();
+    @Override
+    protected void startInternal() throws LifecycleException {
 
         // Validate that we can open our connection - but let tomcat
         // startup in case the database is temporarily unavailable
@@ -798,19 +715,20 @@ public class JDBCRealm
             containerLog.error(sm.getString("jdbcRealm.open"), e);
         }
 
+        super.startInternal();
     }
 
 
     /**
-     * Gracefully shut down active use of the public methods of this Component.
+     * Gracefully terminate the active use of the public methods of this component and implement the requirements of
+     * {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
      *
-     * @exception LifecycleException if this component detects a fatal error
-     *  that needs to be reported
+     * @exception LifecycleException if this component detects a fatal error that needs to be reported
      */
-    public void stop() throws LifecycleException {
+    @Override
+    protected void stopInternal() throws LifecycleException {
 
-        // Perform normal superclass finalization
-        super.stop();
+        super.stopInternal();
 
         // Close any open DB connection
         close(this.dbConnection);
