@@ -19,20 +19,16 @@ package org.apache.catalina.ssi;
 import java.io.IOException;
 import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
-import java.text.DateFormat;
-import java.text.SimpleDateFormat;
 import java.util.Locale;
-import java.util.TimeZone;
 
 import javax.servlet.ServletOutputStream;
 import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpServletResponseWrapper;
 
-import org.apache.tomcat.util.ExceptionUtils;
+import org.apache.tomcat.util.http.FastHttpDateFormat;
 
 /**
- * An HttpServletResponseWrapper, used from
- * <code>SSIServletExternalResolver</code>
+ * An HttpServletResponseWrapper, used from <code>SSIServletExternalResolver</code>
  *
  * @author Bip Thelin
  * @author David Becker
@@ -42,8 +38,6 @@ public class ResponseIncludeWrapper extends HttpServletResponseWrapper {
      * The names of some headers we want to capture.
      */
     private static final String LAST_MODIFIED = "last-modified";
-    private static final DateFormat RFC1123_FORMAT;
-    private static final String RFC1123_PATTERN = "EEE, dd MMM yyyy HH:mm:ss z";
 
     protected long lastModified = -1;
 
@@ -54,29 +48,22 @@ public class ResponseIncludeWrapper extends HttpServletResponseWrapper {
     protected ServletOutputStream servletOutputStream;
     protected PrintWriter printWriter;
 
-    static {
-        RFC1123_FORMAT = new SimpleDateFormat(RFC1123_PATTERN, Locale.US);
-        RFC1123_FORMAT.setTimeZone(TimeZone.getTimeZone("GMT"));
-    }
-
     /**
-     * Initialize our wrapper with the current HttpServletResponse and
-     * ServletOutputStream.
+     * Initialize our wrapper with the current HttpServletResponse and ServletOutputStream.
      *
-     * @param response The response to use
+     * @param response                   The response to use
      * @param captureServletOutputStream The ServletOutputStream to use
      */
-    public ResponseIncludeWrapper(HttpServletResponse response,
-            ServletOutputStream captureServletOutputStream) {
+    public ResponseIncludeWrapper(HttpServletResponse response, ServletOutputStream captureServletOutputStream) {
         super(response);
         this.captureServletOutputStream = captureServletOutputStream;
     }
 
 
     /**
-     * Flush the servletOutputStream or printWriter ( only one will be non-null )
-     * This must be called after a requestDispatcher.include, since we can't
-     * assume that the included servlet flushed its stream.
+     * Flush the servletOutputStream or printWriter ( only one will be non-null ) This must be called after a
+     * requestDispatcher.include, since we can't assume that the included servlet flushed its stream.
+     *
      * @throws IOException an IO error occurred
      */
     public void flushOutputStreamOrWriter() throws IOException {
@@ -90,21 +77,19 @@ public class ResponseIncludeWrapper extends HttpServletResponseWrapper {
 
 
     /**
-     * Return a printwriter, throws and exception if a OutputStream already
-     * been returned.
+     * Return a printwriter, throws an exception if an OutputStream already been returned.
      *
      * @return a PrintWriter object
-     * @exception java.io.IOException
-     *                if the outputstream already been called
+     *
+     * @exception java.io.IOException if the outputstream already been called
      */
     @Override
     public PrintWriter getWriter() throws IOException {
         if (servletOutputStream == null) {
             if (printWriter == null) {
                 setCharacterEncoding(getCharacterEncoding());
-                printWriter = new PrintWriter(
-                        new OutputStreamWriter(captureServletOutputStream,
-                                               getCharacterEncoding()));
+                printWriter =
+                        new PrintWriter(new OutputStreamWriter(captureServletOutputStream, getCharacterEncoding()));
             }
             return printWriter;
         }
@@ -113,12 +98,11 @@ public class ResponseIncludeWrapper extends HttpServletResponseWrapper {
 
 
     /**
-     * Return a OutputStream, throws and exception if a printwriter already
-     * been returned.
+     * Return an OutputStream, throws an exception if a printwriter already been returned.
      *
-     * @return a OutputStream object
-     * @exception java.io.IOException
-     *                if the printwriter already been called
+     * @return an OutputStream object
+     *
+     * @exception java.io.IOException if the printwriter already been called
      */
     @Override
     public ServletOutputStream getOutputStream() throws IOException {
@@ -133,12 +117,11 @@ public class ResponseIncludeWrapper extends HttpServletResponseWrapper {
 
 
     /**
-     * Returns the value of the <code>last-modified</code> header field. The
-     * result is the number of milliseconds since January 1, 1970 GMT.
+     * Returns the value of the <code>last-modified</code> header field. The result is the number of milliseconds since
+     * January 1, 1970 GMT.
      *
-     * @return the date the resource referenced by this
-     *   <code>ResponseIncludeWrapper</code> was last modified, or -1 if not
-     *   known.
+     * @return the date the resource referenced by this <code>ResponseIncludeWrapper</code> was last modified, or -1 if
+     *             not known.
      */
     public long getLastModified() {
         return lastModified;
@@ -158,12 +141,9 @@ public class ResponseIncludeWrapper extends HttpServletResponseWrapper {
         super.addHeader(name, value);
         String lname = name.toLowerCase(Locale.ENGLISH);
         if (lname.equals(LAST_MODIFIED)) {
-            try {
-                synchronized(RFC1123_FORMAT) {
-                    lastModified = RFC1123_FORMAT.parse(value).getTime();
-                }
-            } catch (Throwable ignore) {
-                ExceptionUtils.handleThrowable(ignore);
+            long lastModified = FastHttpDateFormat.parseDate(value);
+            if (lastModified != -1) {
+                this.lastModified = lastModified;
             }
         }
     }
@@ -182,12 +162,9 @@ public class ResponseIncludeWrapper extends HttpServletResponseWrapper {
         super.setHeader(name, value);
         String lname = name.toLowerCase(Locale.ENGLISH);
         if (lname.equals(LAST_MODIFIED)) {
-            try {
-                synchronized(RFC1123_FORMAT) {
-                    lastModified = RFC1123_FORMAT.parse(value).getTime();
-                }
-            } catch (Throwable ignore) {
-                ExceptionUtils.handleThrowable(ignore);
+            long lastModified = FastHttpDateFormat.parseDate(value);
+            if (lastModified != -1) {
+                this.lastModified = lastModified;
             }
         }
     }
