@@ -22,6 +22,7 @@ import java.io.UnsupportedEncodingException;
 import java.net.URL;
 import java.net.URLConnection;
 import java.nio.charset.Charset;
+import java.nio.charset.StandardCharsets;
 import java.util.Collection;
 import java.util.Date;
 import java.util.Enumeration;
@@ -35,7 +36,6 @@ import javax.servlet.http.HttpServletResponse;
 
 import org.apache.catalina.connector.Connector;
 import org.apache.catalina.connector.Request;
-import org.apache.coyote.Constants;
 import org.apache.tomcat.util.buf.B2CConverter;
 import org.apache.tomcat.util.buf.UDecoder;
 import org.apache.tomcat.util.http.RequestUtil;
@@ -49,7 +49,7 @@ import org.apache.tomcat.util.res.StringManager;
  */
 public class SSIServletExternalResolver implements SSIExternalResolver {
     private static final StringManager sm = StringManager.getManager(SSIServletExternalResolver.class);
-    protected final String VARIABLE_NAMES[] = { "AUTH_TYPE", "CONTENT_LENGTH", "CONTENT_TYPE", "DOCUMENT_NAME",
+    protected final String[] VARIABLE_NAMES = { "AUTH_TYPE", "CONTENT_LENGTH", "CONTENT_TYPE", "DOCUMENT_NAME",
             "DOCUMENT_URI", "GATEWAY_INTERFACE", "HTTP_ACCEPT", "HTTP_ACCEPT_ENCODING", "HTTP_ACCEPT_LANGUAGE",
             "HTTP_CONNECTION", "HTTP_HOST", "HTTP_REFERER", "HTTP_USER_AGENT", "PATH_INFO", "PATH_TRANSLATED",
             "QUERY_STRING", "QUERY_STRING_UNESCAPED", "REMOTE_ADDR", "REMOTE_HOST", "REMOTE_PORT", "REMOTE_USER",
@@ -141,7 +141,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
 
     @Override
     public String getVariableValue(String name) {
-        String retVal = null;
+        String retVal;
         Object object = getReqAttributeIgnoreCase(name);
         if (object != null) {
             retVal = object.toString();
@@ -262,7 +262,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
                             queryStringCharset = uriCharset;
                         } else {
                             // Use default as a last resort
-                            queryStringCharset = Constants.DEFAULT_URI_CHARSET;
+                            queryStringCharset = StandardCharsets.UTF_8;
                         }
 
                         retVal = UDecoder.URLDecode(queryString, queryStringCharset);
@@ -309,14 +309,8 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
             } else if (nameParts[1].equals("PROTOCOL")) {
                 retVal = req.getProtocol();
             } else if (nameParts[1].equals("SOFTWARE")) {
-                StringBuilder rv = new StringBuilder(context.getServerInfo());
-                rv.append(' ');
-                rv.append(System.getProperty("java.vm.name"));
-                rv.append('/');
-                rv.append(System.getProperty("java.vm.version"));
-                rv.append(' ');
-                rv.append(System.getProperty("os.name"));
-                retVal = rv.toString();
+                retVal = context.getServerInfo() + ' ' + System.getProperty("java.vm.name") + '/' +
+                        System.getProperty("java.vm.version") + ' ' + System.getProperty("os.name");
             }
         } else if (name.equalsIgnoreCase("UNIQUE_ID")) {
             retVal = req.getRequestedSessionId();
@@ -385,9 +379,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
             throw new IOException(
                     sm.getString("ssiServletExternalResolver.pathTraversalNonVirtualPath", nonVirtualPath));
         }
-        String path = getAbsolutePath(nonVirtualPath);
-        ServletContextAndPath csAndP = new ServletContextAndPath(context, path);
-        return csAndP;
+        return new ServletContextAndPath(context, getAbsolutePath(nonVirtualPath));
     }
 
 
@@ -427,16 +419,14 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
 
 
     protected ServletContextAndPath getServletContextAndPath(String originalPath, boolean virtual) throws IOException {
-        ServletContextAndPath csAndP = null;
         if (debug > 0) {
             log("SSIServletExternalResolver.getServletContextAndPath( " + originalPath + ", " + virtual + ")", null);
         }
         if (virtual) {
-            csAndP = getServletContextAndPathFromVirtualPath(originalPath);
+            return getServletContextAndPathFromVirtualPath(originalPath);
         } else {
-            csAndP = getServletContextAndPathFromNonVirtualPath(originalPath);
+            return getServletContextAndPathFromNonVirtualPath(originalPath);
         }
-        return csAndP;
     }
 
 
@@ -448,8 +438,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
         if (url == null) {
             throw new IOException(sm.getString("ssiServletExternalResolver.noResource", path));
         }
-        URLConnection urlConnection = url.openConnection();
-        return urlConnection;
+        return url.openConnection();
     }
 
 
@@ -459,7 +448,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
         try {
             URLConnection urlConnection = getURLConnection(path, virtual);
             lastModified = urlConnection.getLastModified();
-        } catch (IOException e) {
+        } catch (IOException ignore) {
             // Ignore this. It will always fail for non-file based includes
         }
         return lastModified;
@@ -472,7 +461,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
         try {
             URLConnection urlConnection = getURLConnection(path, virtual);
             fileSize = urlConnection.getContentLengthLong();
-        } catch (IOException e) {
+        } catch (IOException ignore) {
             // Ignore this. It will always fail for non-file based includes
         }
         return fileSize;
@@ -512,7 +501,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
              * Make an assumption that an empty response is a failure. This is a problem if a truly empty file were
              * included, but not sure how else to tell.
              */
-            if (retVal.equals("") && !req.getMethod().equalsIgnoreCase("HEAD")) {
+            if (retVal.isEmpty() && !req.getMethod().equalsIgnoreCase("HEAD")) {
                 throw new IOException(sm.getString("ssiServletExternalResolver.noFile", path));
             }
             return retVal;
