@@ -815,7 +815,7 @@ public class Tomcat {
         }
         try {
             baseFile = baseFile.getCanonicalFile();
-        } catch (IOException e) {
+        } catch (IOException ioe) {
             baseFile = baseFile.getAbsoluteFile();
         }
         server.setCatalinaBase(baseFile);
@@ -832,7 +832,7 @@ public class Tomcat {
             }
             try {
                 homeFile = homeFile.getCanonicalFile();
-            } catch (IOException e) {
+            } catch (IOException ioe) {
                 homeFile = homeFile.getAbsoluteFile();
             }
             server.setCatalinaHome(homeFile);
@@ -996,7 +996,6 @@ public class Tomcat {
      * <li>MIME mappings (subset of those in conf/web.xml)</li>
      * <li>Welcome files</li>
      * </ul>
-     * TODO: Align the MIME mappings with conf/web.xml - possibly via a common file.
      *
      * @param contextPath The path of the context to set the defaults for
      */
@@ -1038,11 +1037,15 @@ public class Tomcat {
         ctx.addWelcomeFile("index.html");
         ctx.addWelcomeFile("index.htm");
         ctx.addWelcomeFile("index.jsp");
+        // Any application configured welcome files should override the defaults.
+        if (ctx instanceof StandardContext) {
+            ((StandardContext) ctx).setReplaceWelcomeFiles(true);
+        }
     }
 
 
     /**
-     * Add the default MIME type mappings to the provide Context.
+     * Add the default MIME type mappings to the provided Context.
      *
      * @param context The web application to which the default MIME type mappings should be added.
      */
@@ -1053,8 +1056,8 @@ public class Tomcat {
             for (Map.Entry<Object,Object> entry : defaultMimeMappings.entrySet()) {
                 context.addMimeMapping((String) entry.getKey(), (String) entry.getValue());
             }
-        } catch (IOException e) {
-            throw new IllegalStateException(sm.getString("tomcat.defaultMimeTypeMappingsFail"), e);
+        } catch (IOException ioe) {
+            throw new IllegalStateException(sm.getString("tomcat.defaultMimeTypeMappingsFail"), ioe);
         }
     }
 
@@ -1206,9 +1209,9 @@ public class Tomcat {
             if (entry != null) {
                 result = UriUtil.buildJarUrl(docBase, Constants.ApplicationContextXml);
             }
-        } catch (IOException e) {
+        } catch (IOException ioe) {
             Logger.getLogger(getLoggerName(getHost(), contextName)).log(Level.WARNING,
-                    sm.getString("tomcat.noContextXml", docBase), e);
+                    sm.getString("tomcat.noContextXml", docBase), ioe);
         }
         return result;
     }
