@@ -32,7 +32,6 @@ import java.security.cert.Certificate;
 import java.text.Collator;
 import java.util.Arrays;
 import java.util.Locale;
-import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
 import java.util.jar.Manifest;
 
@@ -40,7 +39,9 @@ import org.apache.catalina.WebResource;
 import org.apache.catalina.WebResourceRoot;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.buf.HexUtils;
 import org.apache.tomcat.util.res.StringManager;
+import org.apache.tomcat.util.security.ConcurrentMessageDigest;
 
 /**
  * This class is designed to wrap a 'raw' WebResource and providing caching for expensive operations. Inexpensive
@@ -74,6 +75,7 @@ public class CachedResource implements WebResource {
     private volatile Boolean cachedExists = null;
     private volatile Boolean cachedIsVirtual = null;
     private volatile Long cachedContentLength = null;
+    private volatile String cachedStrongETag = null;
 
 
     public CachedResource(Cache cache, StandardRoot root, String path, long ttl, int objectMaxSizeBytes,
@@ -82,6 +84,7 @@ public class CachedResource implements WebResource {
         this.root = root;
         this.webAppPath = path;
         this.ttl = ttl;
+        nextCheck = ttl + System.currentTimeMillis();
         this.objectMaxSizeBytes = objectMaxSizeBytes;
         this.usesClassLoaderResources = usesClassLoaderResources;
     }
@@ -98,15 +101,12 @@ public class CachedResource implements WebResource {
             return false;
         }
 
-        long now = System.currentTimeMillis();
-
         if (webResource == null) {
             synchronized (this) {
                 if (webResource == null) {
                     webResource = root.getResourceInternal(webAppPath, useClassLoaderResources);
                     getLastModified();
                     getContentLength();
-                    nextCheck = ttl + now;
                     // exists() is a relatively expensive check for a file so
                     // use the fact that we know if it exists at this point
                     if (webResource instanceof EmptyResource) {
@@ -119,6 +119,8 @@ public class CachedResource implements WebResource {
             }
         }
 
+        long now = System.currentTimeMillis();
+
         if (now < nextCheck) {
             return true;
         }
@@ -244,13 +246,26 @@ public class CachedResource implements WebResource {
 
     @Override
     public long getContentLength() {
+        /*
+         * Cache the content length for two reasons.
+         *
+         * 1. It is relatively expensive to calculate, it shouldn't change and this method is called multiple times
+         * during cache validation. Caching, therefore, offers a performance benefit.
+         *
+         * 2. There is a race condition if concurrent threads are trying to PUT and DELETE the same resource. See BZ
+         * 69527 (https://bz.apache.org/bugzilla/show_bug.cgi?id=69527#c14) for full details. The short version is that
+         * getContentLength() must always return the same value for any one CachedResource instance else the cache size
+         * will be corrupted.
+         */
         if (cachedContentLength == null) {
-            long result = 0;
-            if (webResource != null) {
-                result = webResource.getContentLength();
-                cachedContentLength = Long.valueOf(result);
+            if (webResource == null) {
+                synchronized (this) {
+                    if (webResource == null) {
+                        webResource = root.getResourceInternal(webAppPath, usesClassLoaderResources);
+                    }
+                }
             }
-            return result;
+            cachedContentLength = Long.valueOf(webResource.getContentLength());
         }
         return cachedContentLength.longValue();
     }
@@ -275,6 +290,20 @@ public class CachedResource implements WebResource {
         return webResource.getETag();
     }
 
+    @Override
+    public String getStrongETag() {
+        if (cachedStrongETag == null) {
+            byte[] buf = getContent();
+            if (buf != null) {
+                buf = ConcurrentMessageDigest.digest("SHA-1", buf);
+                cachedStrongETag = "\"" + HexUtils.toHexString(buf) + "\"";
+            } else {
+                cachedStrongETag = webResource.getStrongETag();
+            }
+        }
+        return cachedStrongETag;
+    }
+
     @Override
     public void setMimeType(String mimeType) {
         webResource.setMimeType(mimeType);
@@ -327,7 +356,7 @@ public class CachedResource implements WebResource {
          *
          * One option to resolve this issue is to use a custom URL scheme for resource URLs. This would allow us, via
          * registration of a URLStreamHandlerFactory, to control how the resources are accessed and ensure that all
-         * access go via the cache We took this approach for war: URLs so we can use jar:war:file: URLs to reference
+         * access go via the cache. We took this approach for war: URLs so we can use jar:war:file: URLs to reference
          * resources in unpacked WAR files. However, because URL.setURLStreamHandlerFactory() may only be called once,
          * this can cause problems when using other libraries that also want to use a custom URL scheme.
          *
@@ -340,8 +369,8 @@ public class CachedResource implements WebResource {
             return null;
         }
         try {
-            CachedResourceURLStreamHandler handler = new CachedResourceURLStreamHandler(resourceURL, root, webAppPath,
-                    usesClassLoaderResources);
+            CachedResourceURLStreamHandler handler =
+                    new CachedResourceURLStreamHandler(resourceURL, root, webAppPath, usesClassLoaderResources);
             URL result = new URL(null, resourceURL.toExternalForm(), handler);
             handler.setCacheURL(result);
             return result;
@@ -392,7 +421,7 @@ public class CachedResource implements WebResource {
         // Longer paths use a noticeable amount of memory so account for this in
         // the cache size. The fixed component of a String instance's memory
         // usage is accounted for in the 500 bytes above.
-        result += getWebappPath().length() * 2;
+        result += getWebappPath().length() * 2L;
         if (getContentLength() <= objectMaxSizeBytes) {
             result += getContentLength();
         }
@@ -562,6 +591,12 @@ public class CachedResource implements WebResource {
         private WebResource getResource() {
             return root.getResource(webAppPath, false, usesClassLoaderResources);
         }
+
+        @Override
+        public String getContentType() {
+            // "content/unknown" is the value used by sun.net.www.URLConnection. It is used here for consistency.
+            return (getResource().getMimeType() != null) ? getResource().getMimeType() : "content/unknown";
+        }
     }
 
 
@@ -624,13 +659,5 @@ public class CachedResource implements WebResource {
             return ((JarURLConnection) resourceURL.openConnection()).getJarFile();
         }
 
-        @Override
-        public JarEntry getJarEntry() throws IOException {
-            if (getEntryName() == null) {
-                return null;
-            } else {
-                return super.getJarEntry();
-            }
-        }
     }
 }
