@@ -66,23 +66,15 @@ class Validator {
 
         private final ErrorDispatcher err;
 
-        private static final JspUtil.ValidAttribute[] pageDirectiveAttrs = {
-            new JspUtil.ValidAttribute("language"),
-            new JspUtil.ValidAttribute("extends"),
-            new JspUtil.ValidAttribute("import"),
-            new JspUtil.ValidAttribute("session"),
-            new JspUtil.ValidAttribute("buffer"),
-            new JspUtil.ValidAttribute("autoFlush"),
-            new JspUtil.ValidAttribute("isThreadSafe"),
-            new JspUtil.ValidAttribute("info"),
-            new JspUtil.ValidAttribute("errorPage"),
-            new JspUtil.ValidAttribute("isErrorPage"),
-            new JspUtil.ValidAttribute("contentType"),
-            new JspUtil.ValidAttribute("pageEncoding"),
-            new JspUtil.ValidAttribute("isELIgnored"),
-            new JspUtil.ValidAttribute("deferredSyntaxAllowedAsLiteral"),
-            new JspUtil.ValidAttribute("trimDirectiveWhitespaces")
-        };
+        private static final JspUtil.ValidAttribute[] pageDirectiveAttrs = { new JspUtil.ValidAttribute("language"),
+                new JspUtil.ValidAttribute("extends"), new JspUtil.ValidAttribute("import"),
+                new JspUtil.ValidAttribute("session"), new JspUtil.ValidAttribute("buffer"),
+                new JspUtil.ValidAttribute("autoFlush"), new JspUtil.ValidAttribute("isThreadSafe"),
+                new JspUtil.ValidAttribute("info"), new JspUtil.ValidAttribute("errorPage"),
+                new JspUtil.ValidAttribute("isErrorPage"), new JspUtil.ValidAttribute("contentType"),
+                new JspUtil.ValidAttribute("pageEncoding"), new JspUtil.ValidAttribute("isELIgnored"),
+                new JspUtil.ValidAttribute("deferredSyntaxAllowedAsLiteral"),
+                new JspUtil.ValidAttribute("trimDirectiveWhitespaces") };
 
         private boolean pageEncodingSeen = false;
 
@@ -444,8 +436,7 @@ class Validator {
         public void visit(Node.JspRoot n) throws JasperException {
             JspUtil.checkAttributes("Jsp:root", n, jspRootAttrs, err);
             String version = n.getTextAttribute("version");
-            if (!version.equals("1.2") && !version.equals("2.0") &&
-                    !version.equals("2.1") && !version.equals("2.2") &&
+            if (!version.equals("1.2") && !version.equals("2.0") && !version.equals("2.1") && !version.equals("2.2") &&
                     !version.equals("2.3")) {
                 err.jspError(n, "jsp.error.jsproot.version.invalid", version);
             }
@@ -715,10 +706,8 @@ class Validator {
                 char c = value.charAt(i);
                 if (c == '#' && (i + 1) < len && value.charAt(i + 1) == '{' && !prevCharIsEscape) {
                     return true;
-                } else if (c == '\\') {
-                    prevCharIsEscape = true;
                 } else {
-                    prevCharIsEscape = false;
+                    prevCharIsEscape = c == '\\';
                 }
                 i++;
             }
@@ -1020,7 +1009,7 @@ class Validator {
                 }
                 for (int j = 0; tldAttrs != null && j < tldAttrs.length; j++) {
                     if (attrs.getLocalName(i).equals(tldAttrs[j].getName()) && (attrs.getURI(i) == null ||
-                            attrs.getURI(i).length() == 0 || attrs.getURI(i).equals(n.getURI()))) {
+                            attrs.getURI(i).isEmpty() || attrs.getURI(i).equals(n.getURI()))) {
 
                         TagAttributeInfo tldAttr = tldAttrs[j];
                         if (tldAttr.canBeRequestTime() || tldAttr.isDeferredMethod() || tldAttr.isDeferredValue()) { // JSP
@@ -1149,7 +1138,7 @@ class Validator {
                      */
                     String attrPrefix = na.getPrefix();
                     if (na.getLocalName().equals(tldAttr.getName()) &&
-                            (attrPrefix == null || attrPrefix.length() == 0 || attrPrefix.equals(n.getPrefix()))) {
+                            (attrPrefix == null || attrPrefix.isEmpty() || attrPrefix.equals(n.getPrefix()))) {
                         jspAttrs[start + i] = new Node.JspAttribute(na, tldAttr, false);
                         NamedAttributeVisitor nav = null;
                         if (na.getBody() != null) {
@@ -1178,10 +1167,10 @@ class Validator {
         /**
          * Preprocess attributes that can be expressions. Expression delimiters are stripped.
          * <p>
-         * If value is null, checks if there are any NamedAttribute subelements in the tree node, and if so, constructs
+         * If value is null, checks if there are any NamedAttribute sub elements in the tree node, and if so, constructs
          * a JspAttribute out of a child NamedAttribute node.
          *
-         * @param el EL expression, if already parsed by the caller (so that we can skip re-parsing it)
+         * @param el EL expression, if already parsed by the caller (so that we can skip reparsing it)
          */
         private Node.JspAttribute getJspAttribute(TagAttributeInfo tai, String qName, String uri, String localName,
                 String value, Node n, ELNode.Nodes el, boolean dynamic) throws JasperException {
@@ -1379,7 +1368,7 @@ class Validator {
 
             class FVVisitor extends ELNode.Visitor {
 
-                private Node n;
+                private final Node n;
 
                 FVVisitor(Node n) {
                     this.n = n;
@@ -1498,7 +1487,7 @@ class Validator {
 
             class ValidateFunctionMapper extends FunctionMapper {
 
-                private Map<String,Method> fnmap = new HashMap<>();
+                private final Map<String,Method> fnmap = new HashMap<>();
 
                 @Override
                 public void mapFunction(String prefix, String localName, Method method) {
@@ -1512,7 +1501,7 @@ class Validator {
             }
 
             class MapperELVisitor extends ELNode.Visitor {
-                private ValidateFunctionMapper fmapper;
+                private final ValidateFunctionMapper fmapper;
 
                 MapperELVisitor(ValidateFunctionMapper fmapper) {
                     this.fmapper = fmapper;
@@ -1535,9 +1524,9 @@ class Validator {
                         err.jspError("jsp.error.function.classnotfound", n.getFunctionInfo().getFunctionClass(),
                                 n.getPrefix() + ':' + n.getName(), e.getMessage());
                     }
-                    String paramTypes[] = n.getParameters();
+                    String[] paramTypes = n.getParameters();
                     int size = paramTypes.length;
-                    Class<?> params[] = new Class[size];
+                    Class<?>[] params = new Class[size];
                     int i = 0;
                     try {
                         for (i = 0; i < size; i++) {
