@@ -35,10 +35,11 @@ import javax.servlet.jsp.tagext.TagInfo;
 import org.apache.jasper.JasperException;
 import org.apache.jasper.JspCompilationContext;
 import org.apache.jasper.Options;
-import org.apache.jasper.compiler.ErrorDispatcher;
 import org.apache.jasper.compiler.JavacErrorDetail;
 import org.apache.jasper.compiler.JspRuntimeContext;
 import org.apache.jasper.compiler.Localizer;
+import org.apache.jasper.compiler.SmapInput;
+import org.apache.jasper.compiler.SmapStratum;
 import org.apache.jasper.runtime.ExceptionUtils;
 import org.apache.jasper.runtime.InstanceManagerFactory;
 import org.apache.jasper.runtime.JspSourceDependent;
@@ -49,14 +50,9 @@ import org.apache.tomcat.InstanceManager;
 import org.apache.tomcat.Jar;
 
 /**
- * The JSP engine (a.k.a Jasper).
- *
- * The servlet container is responsible for providing a
- * URLClassLoader for the web application context Jasper
- * is being used in. Jasper will try get the Tomcat
- * ServletContext attribute for its ServletContext class
- * loader, if that fails, it uses the parent class loader.
- * In either case, it must be a URLClassLoader.
+ * The Jasper JSP engine. The servlet container is responsible for providing a URLClassLoader for the web application
+ * context Jasper is being used in. Jasper will try to get the Tomcat ServletContext attribute for its ServletContext
+ * class loader, if that fails, it uses the parent class loader. In either case, it must be a URLClassLoader.
  *
  * @author Anil K. Vijendran
  * @author Harish Prabandham
@@ -69,8 +65,7 @@ import org.apache.tomcat.Jar;
 @SuppressWarnings("deprecation") // Have to support SingleThreadModel
 public class JspServletWrapper {
 
-    private static final Map<String,Long> ALWAYS_OUTDATED_DEPENDENCIES =
-            new HashMap<>();
+    private static final Map<String,Long> ALWAYS_OUTDATED_DEPENDENCIES = new HashMap<>();
 
     static {
         // If this is missing,
@@ -88,11 +83,9 @@ public class JspServletWrapper {
     private final ServletConfig config;
     private final Options options;
     /*
-     * The servlet / tag file needs a compilation check on first access. Use a
-     * separate flag (rather then theServlet == null / tagHandlerClass == null
-     * as it avoids the potentially expensive isOutDated() calls in
-     * ctxt.compile() if there are multiple concurrent requests for the servlet
-     * / tag before the class has been loaded.
+     * The servlet / tag file needs a compilation check on first access. Use a separate flag (rather than theServlet ==
+     * null / tagHandlerClass == null) as it avoids the potentially expensive isOutDated() calls in ctxt.compile() if
+     * there are multiple concurrent requests for the servlet / tag before the class has been loaded.
      */
     private volatile boolean mustCompile = true;
     /* Whether the servlet/tag file needs reloading on next access */
@@ -112,42 +105,33 @@ public class JspServletWrapper {
     /*
      * JspServletWrapper for JSP pages.
      */
-    public JspServletWrapper(ServletConfig config, Options options,
-            String jspUri, JspRuntimeContext rctxt) {
+    public JspServletWrapper(ServletConfig config, Options options, String jspUri, JspRuntimeContext rctxt) {
 
         this.isTagFile = false;
         this.config = config;
         this.options = options;
         this.jspUri = jspUri;
-        unloadByCount = options.getMaxLoadedJsps() > 0 ? true : false;
-        unloadByIdle = options.getJspIdleTimeout() > 0 ? true : false;
-        unloadAllowed = unloadByCount || unloadByIdle ? true : false;
-        ctxt = new JspCompilationContext(jspUri, options,
-                                         config.getServletContext(),
-                                         this, rctxt);
+        unloadByCount = options.getMaxLoadedJsps() > 0;
+        unloadByIdle = options.getJspIdleTimeout() > 0;
+        unloadAllowed = unloadByCount || unloadByIdle;
+        ctxt = new JspCompilationContext(jspUri, options, config.getServletContext(), this, rctxt);
     }
 
     /*
      * JspServletWrapper for tag files.
      */
-    public JspServletWrapper(ServletContext servletContext,
-                             Options options,
-                             String tagFilePath,
-                             TagInfo tagInfo,
-                             JspRuntimeContext rctxt,
-                             Jar tagJar) {
+    public JspServletWrapper(ServletContext servletContext, Options options, String tagFilePath, TagInfo tagInfo,
+            JspRuntimeContext rctxt, Jar tagJar) {
 
         this.isTagFile = true;
-        this.config = null;        // not used
+        this.config = null; // not used
         this.options = options;
         this.jspUri = tagFilePath;
         this.tripCount = 0;
-        unloadByCount = options.getMaxLoadedJsps() > 0 ? true : false;
-        unloadByIdle = options.getJspIdleTimeout() > 0 ? true : false;
-        unloadAllowed = unloadByCount || unloadByIdle ? true : false;
-        ctxt = new JspCompilationContext(jspUri, tagInfo, options,
-                                         servletContext, this, rctxt,
-                                         tagJar);
+        unloadByCount = options.getMaxLoadedJsps() > 0;
+        unloadByIdle = options.getJspIdleTimeout() > 0;
+        unloadAllowed = unloadByCount || unloadByIdle;
+        ctxt = new JspCompilationContext(jspUri, tagInfo, options, servletContext, this, rctxt, tagJar);
     }
 
     public JspCompilationContext getJspEngineContext() {
@@ -168,13 +152,11 @@ public class JspServletWrapper {
 
     public Servlet getServlet() throws ServletException {
         /*
-         * DCL on 'reload' requires that 'reload' be volatile
-         * (this also forces a read memory barrier, ensuring the new servlet
-         * object is read consistently).
+         * DCL on 'reload' requires that 'reload' be volatile (this also forces a read memory barrier, ensuring the new
+         * servlet object is read consistently).
          *
-         * When running in non development mode with a checkInterval it is
-         * possible (see BZ 62603) for a race condition to cause failures
-         * if a Servlet or tag is reloaded while a compile check is running
+         * When running in non development mode with a checkInterval it is possible (see BZ 62603) for a race condition
+         * to cause failures if a Servlet or tag is reloaded while a compile check is running
          */
         if (getReloadInternal() || theServlet == null) {
             synchronized (this) {
@@ -190,8 +172,7 @@ public class JspServletWrapper {
                         InstanceManager instanceManager = InstanceManagerFactory.getInstanceManager(config);
                         servlet = (Servlet) instanceManager.newInstance(ctxt.getFQCN(), ctxt.getJspLoader());
                     } catch (Exception e) {
-                        Throwable t = ExceptionUtils
-                                .unwrapInvocationTargetException(e);
+                        Throwable t = ExceptionUtils.unwrapInvocationTargetException(e);
                         ExceptionUtils.handleThrowable(t);
                         throw new JasperException(t);
                     }
@@ -225,8 +206,7 @@ public class JspServletWrapper {
     }
 
     /**
-     * Sets the last-modified time of the servlet class file associated with
-     * this JspServletWrapper.
+     * Sets the last-modified time of the servlet class file associated with this JspServletWrapper.
      *
      * @param lastModified Last-modified time of servlet class
      */
@@ -244,7 +224,7 @@ public class JspServletWrapper {
                     // a new loader will be created which will load the new
                     // class.
                     // TODO Are there inefficiencies between reload and the
-                    //      isOutDated() check?
+                    // isOutDated() check?
                     ctxt.clearJspLoader();
                 }
             }
@@ -253,7 +233,9 @@ public class JspServletWrapper {
 
     /**
      * Compile (if needed) and load a tag file.
+     *
      * @return the loaded class
+     *
      * @throws JasperException Error compiling or loading tag file
      */
     public Class<?> loadTagFile() throws JasperException {
@@ -292,11 +274,11 @@ public class JspServletWrapper {
     }
 
     /**
-     * Compile and load a prototype for the Tag file.  This is needed
-     * when compiling tag files with circular dependencies.  A prototype
-     * (skeleton) with no dependencies on other other tag files is
-     * generated and compiled.
+     * Compile and load a prototype for the Tag file. This is needed when compiling tag files with circular
+     * dependencies. A prototype (skeleton) with no dependencies on other tag files is generated and compiled.
+     *
      * @return the loaded class
+     *
      * @throws JasperException Error compiling or loading tag file
      */
     public Class<?> loadTagFilePrototype() throws JasperException {
@@ -311,9 +293,10 @@ public class JspServletWrapper {
 
     /**
      * Get a list of files that the current page has source dependency on.
+     *
      * @return the map of dependent resources
      */
-    public java.util.Map<String,Long> getDependants() {
+    public Map<String,Long> getDependants() {
         try {
             Object target;
             if (isTagFile) {
@@ -362,9 +345,7 @@ public class JspServletWrapper {
         return unloadHandle;
     }
 
-    public void service(HttpServletRequest request,
-                        HttpServletResponse response,
-                        boolean precompile)
+    public void service(HttpServletRequest request, HttpServletResponse response, boolean precompile)
             throws ServletException, IOException, FileNotFoundException {
 
         Servlet servlet;
@@ -378,9 +359,8 @@ public class JspServletWrapper {
             if ((available > 0L) && (available < Long.MAX_VALUE)) {
                 if (available > System.currentTimeMillis()) {
                     response.setDateHeader("Retry-After", available);
-                    response.sendError
-                        (HttpServletResponse.SC_SERVICE_UNAVAILABLE,
-                         Localizer.getMessage("jsp.error.unavailable"));
+                    response.sendError(HttpServletResponse.SC_SERVICE_UNAVAILABLE,
+                            Localizer.getMessage("jsp.error.unavailable"));
                     return;
                 }
 
@@ -424,11 +404,11 @@ public class JspServletWrapper {
                 throw handleJspException(ex);
             }
             throw ex;
-        } catch (Exception ex) {
+        } catch (Exception e) {
             if (options.getDevelopment()) {
-                throw handleJspException(ex);
+                throw handleJspException(e);
             }
-            throw new JasperException(ex);
+            throw new JasperException(e);
         }
 
         try {
@@ -436,7 +416,7 @@ public class JspServletWrapper {
              * (3) Handle limitation of number of loaded Jsps
              */
             if (unloadAllowed) {
-                synchronized(this) {
+                synchronized (this) {
                     if (unloadByCount) {
                         if (unloadHandle == null) {
                             unloadHandle = ctxt.getRuntimeContext().push(this);
@@ -456,17 +436,16 @@ public class JspServletWrapper {
              * (4) Service request
              */
             if (servlet instanceof SingleThreadModel) {
-               // sync on the wrapper so that the freshness
-               // of the page is determined right before servicing
-               synchronized (this) {
-                   servlet.service(request, response);
+                // sync on the wrapper so that the freshness
+                // of the page is determined right before servicing
+                synchronized (this) {
+                    servlet.service(request, response);
                 }
             } else {
                 servlet.service(request, response);
             }
         } catch (UnavailableException ex) {
-            String includeRequestUri = (String)
-                request.getAttribute(RequestDispatcher.INCLUDE_REQUEST_URI);
+            String includeRequestUri = (String) request.getAttribute(RequestDispatcher.INCLUDE_REQUEST_URI);
             if (includeRequestUri != null) {
                 // This file was included. Throw an exception as
                 // a response.sendError() will be ignored by the
@@ -476,28 +455,25 @@ public class JspServletWrapper {
 
             int unavailableSeconds = ex.getUnavailableSeconds();
             if (unavailableSeconds <= 0) {
-                unavailableSeconds = 60;        // Arbitrary default
+                unavailableSeconds = 60; // Arbitrary default
             }
-            available = System.currentTimeMillis() +
-                (unavailableSeconds * 1000L);
-            response.sendError
-                (HttpServletResponse.SC_SERVICE_UNAVAILABLE,
-                 ex.getMessage());
+            available = System.currentTimeMillis() + (unavailableSeconds * 1000L);
+            response.sendError(HttpServletResponse.SC_SERVICE_UNAVAILABLE, ex.getMessage());
         } catch (ServletException | IllegalStateException ex) {
-            if(options.getDevelopment()) {
-                throw handleJspException(ex);
-            }
-            throw ex;
-        } catch (IOException ex) {
             if (options.getDevelopment()) {
-                throw new IOException(handleJspException(ex).getMessage(), ex);
-            }
-            throw ex;
-        } catch (Exception ex) {
-            if(options.getDevelopment()) {
                 throw handleJspException(ex);
             }
-            throw new JasperException(ex);
+            throw ex;
+        } catch (IOException ioe) {
+            if (options.getDevelopment()) {
+                throw new IOException(handleJspException(ioe).getMessage(), ioe);
+            }
+            throw ioe;
+        } catch (Exception e) {
+            if (options.getDevelopment()) {
+                throw handleJspException(e);
+            }
+            throw new JasperException(e);
         }
     }
 
@@ -516,8 +492,7 @@ public class JspServletWrapper {
                 Throwable t = ExceptionUtils.unwrapInvocationTargetException(e);
                 ExceptionUtils.handleThrowable(t);
                 // Log any exception, since it can't be passed along
-                log.error(Localizer.getMessage("jsp.error.file.not.found",
-                        e.getMessage()), t);
+                log.error(Localizer.getMessage("jsp.error.file.not.found", e.getMessage()), t);
             }
         }
     }
@@ -528,6 +503,7 @@ public class JspServletWrapper {
     public long getLastModificationTest() {
         return lastModificationTest;
     }
+
     /**
      * @param lastModificationTest The lastModificationTest to set.
      */
@@ -543,16 +519,16 @@ public class JspServletWrapper {
     }
 
     /**
-     * <p>Attempts to construct a JasperException that contains helpful information
-     * about what went wrong. Uses the JSP compiler system to translate the line
-     * number in the generated servlet that originated the exception to a line
-     * number in the JSP.  Then constructs an exception containing that
-     * information, and a snippet of the JSP to help debugging.
-     * Please see https://bz.apache.org/bugzilla/show_bug.cgi?id=37062 and
-     * http://www.tfenne.com/jasper/ for more details.
+     * <p>
+     * Attempts to construct a JasperException that contains helpful information about what went wrong. Uses the JSP
+     * compiler system to translate the line number in the generated servlet that originated the exception to a line
+     * number in the JSP. Then constructs an exception containing that information, and a snippet of the JSP to help
+     * debugging. Please see <a href="https://bz.apache.org/bugzilla/show_bug.cgi?id=37062">BZ 37062</a> for more
+     * details.
      * </p>
      *
      * @param ex the exception that was the cause of the problem.
+     *
      * @return a JasperException with more detailed information
      */
     protected JasperException handleJspException(Exception ex) {
@@ -562,55 +538,57 @@ public class JspServletWrapper {
                 realException = ((ServletException) ex).getRootCause();
             }
 
-            // First identify the stack frame in the trace that represents the JSP
+            // Find the first stack frame that represents code generated by
+            // Jasper
             StackTraceElement[] frames = realException.getStackTrace();
             StackTraceElement jspFrame = null;
 
-            for (int i=0; i<frames.length; ++i) {
-                if ( frames[i].getClassName().equals(this.getServlet().getClass().getName()) ) {
-                    jspFrame = frames[i];
+            String servletPackageName = ctxt.getBasePackageName();
+            for (StackTraceElement frame : frames) {
+                if (frame.getClassName().startsWith(servletPackageName)) {
+                    jspFrame = frame;
                     break;
                 }
             }
 
+            SmapStratum smap = null;
 
-            if (jspFrame == null ||
-                    this.ctxt.getCompiler().getPageNodes() == null) {
+            if (jspFrame != null) {
+                smap = ctxt.getCompiler().getSmap(jspFrame.getClassName());
+            }
+
+            if (smap == null) {
                 // If we couldn't find a frame in the stack trace corresponding
-                // to the generated servlet class or we don't have a copy of the
-                // parsed JSP to hand, we can't really add anything
+                // to the generated servlet class, or we don't have a copy of the
+                // smap to hand, we can't really add anything
                 return new JasperException(ex);
             }
 
+            @SuppressWarnings("null")
             int javaLineNumber = jspFrame.getLineNumber();
-            JavacErrorDetail detail = ErrorDispatcher.createJavacError(
-                    jspFrame.getMethodName(),
-                    this.ctxt.getCompiler().getPageNodes(),
-                    null,
-                    javaLineNumber,
-                    ctxt);
+            SmapInput source = smap.getInputLineNumber(javaLineNumber);
 
             // If the line number is less than one we couldn't find out
             // where in the JSP things went wrong
-            int jspLineNumber = detail.getJspBeginLineNumber();
-            if (jspLineNumber < 1) {
+            if (source.getLineNumber() < 1) {
                 throw new JasperException(ex);
             }
 
+            JavacErrorDetail detail = new JavacErrorDetail(jspFrame.getMethodName(), javaLineNumber,
+                    source.getFileName(), source.getLineNumber(), null, ctxt);
+
             if (options.getDisplaySourceFragment()) {
-                return new JasperException(Localizer.getMessage
-                        ("jsp.exception", detail.getJspFileName(),
-                                "" + jspLineNumber) + System.lineSeparator() +
-                                System.lineSeparator() + detail.getJspExtract() +
-                                System.lineSeparator() + System.lineSeparator() +
-                                "Stacktrace:", ex);
+                return new JasperException(
+                        Localizer.getMessage("jsp.exception", detail.getJspFileName(), "" + source.getLineNumber()) +
+                                System.lineSeparator() + System.lineSeparator() + detail.getJspExtract() +
+                                System.lineSeparator() + System.lineSeparator() + "Stacktrace:",
+                        ex);
 
             }
 
-            return new JasperException(Localizer.getMessage
-                    ("jsp.exception", detail.getJspFileName(),
-                            "" + jspLineNumber), ex);
-        } catch (Exception je) {
+            return new JasperException(
+                    Localizer.getMessage("jsp.exception", detail.getJspFileName(), "" + source.getLineNumber()), ex);
+        } catch (Exception e) {
             // If anything goes wrong, just revert to the original behaviour
             if (ex instanceof JasperException) {
                 return (JasperException) ex;
