@@ -85,8 +85,8 @@ public final class Parameters {
             charset = DEFAULT_BODY_CHARSET;
         }
         this.charset = charset;
-        if (log.isDebugEnabled()) {
-            log.debug("Set encoding to " + charset.name());
+        if (log.isTraceEnabled()) {
+            log.trace("Set encoding to " + charset.name());
         }
     }
 
@@ -96,8 +96,8 @@ public final class Parameters {
         }
         this.queryStringCharset = queryStringCharset;
 
-        if (log.isDebugEnabled()) {
-            log.debug("Set query string encoding to " + queryStringCharset.name());
+        if (log.isTraceEnabled()) {
+            log.trace("Set query string encoding to " + queryStringCharset.name());
         }
     }
 
@@ -157,7 +157,7 @@ public final class Parameters {
         handleQueryParameters();
         ArrayList<String> values = paramHashValues.get(name);
         if (values != null) {
-            if (values.size() == 0) {
+            if (values.isEmpty()) {
                 return "";
             }
             return values.get(0);
@@ -181,15 +181,15 @@ public final class Parameters {
             return;
         }
 
-        if (log.isDebugEnabled()) {
-            log.debug("Decoding query " + decodedQuery + " " + queryStringCharset.name());
+        if (log.isTraceEnabled()) {
+            log.trace("Decoding query " + decodedQuery + " " + queryStringCharset.name());
         }
 
         try {
             decodedQuery.duplicate(queryMB);
-        } catch (IOException e) {
+        } catch (IOException ioe) {
             // Can't happen, as decodedQuery can't overflow
-            e.printStackTrace();
+            log.error(sm.getString("parameters.copyFail"), ioe);
         }
         processParameters(decodedQuery, queryStringCharset);
     }
@@ -227,14 +227,14 @@ public final class Parameters {
     private static final Charset DEFAULT_URI_CHARSET = StandardCharsets.UTF_8;
 
 
-    public void processParameters(byte bytes[], int start, int len) {
+    public void processParameters(byte[] bytes, int start, int len) {
         processParameters(bytes, start, len, charset);
     }
 
-    private void processParameters(byte bytes[], int start, int len, Charset charset) {
+    private void processParameters(byte[] bytes, int start, int len, Charset charset) {
 
-        if (log.isDebugEnabled()) {
-            log.debug(sm.getString("parameters.bytes", new String(bytes, start, len, DEFAULT_BODY_CHARSET)));
+        if (log.isTraceEnabled()) {
+            log.trace(sm.getString("parameters.bytes", new String(bytes, start, len, DEFAULT_BODY_CHARSET)));
         }
 
         int decodeFailCount = 0;
@@ -407,14 +407,14 @@ public final class Parameters {
                     }
                     break;
                 }
-            } catch (IOException e) {
+            } catch (IOException ioe) {
                 setParseFailedReason(FailReason.URL_DECODING);
                 decodeFailCount++;
                 if (decodeFailCount == 1 || log.isDebugEnabled()) {
                     if (log.isDebugEnabled()) {
                         log.debug(
                                 sm.getString("parameters.decodeFail.debug", origName.toString(), origValue.toString()),
-                                e);
+                                ioe);
                     } else if (log.isInfoEnabled()) {
                         UserDataHelper.Mode logMode = userDataLog.getNextMode();
                         if (logMode != null) {
@@ -478,7 +478,7 @@ public final class Parameters {
             data.toBytes();
         }
         ByteChunk bc = data.getByteChunk();
-        processParameters(bc.getBytes(), bc.getOffset(), bc.getLength(), charset);
+        processParameters(bc.getBytes(), bc.getStart(), bc.getLength(), charset);
     }
 
     /**
