@@ -17,6 +17,7 @@
 package org.apache.catalina.authenticator;
 
 import java.io.IOException;
+import java.nio.charset.StandardCharsets;
 import java.util.concurrent.atomic.AtomicInteger;
 
 import javax.servlet.http.HttpServletResponse;
@@ -26,8 +27,14 @@ import org.junit.Before;
 import org.junit.Test;
 
 import org.apache.catalina.Context;
+import org.apache.catalina.Engine;
+import org.apache.catalina.Host;
+import org.apache.catalina.Service;
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.core.StandardContext;
+import org.apache.catalina.core.StandardEngine;
+import org.apache.catalina.core.StandardHost;
+import org.apache.catalina.core.StandardService;
 import org.apache.catalina.filters.TesterHttpServletResponse;
 import org.apache.catalina.startup.TesterMapRealm;
 import org.apache.tomcat.util.buf.HexUtils;
@@ -69,8 +76,7 @@ public class TesterDigestAuthenticatorPerformance {
 
         // Create the runnables & threads
         for (int i = 0; i < threadCount; i++) {
-            runnables[i] =
-                    new TesterRunnable(authenticator, nonce, requestCount);
+            runnables[i] = new TesterRunnable(authenticator, nonce, requestCount);
             threads[i] = new Thread(runnables[i]);
         }
 
@@ -91,18 +97,15 @@ public class TesterDigestAuthenticatorPerformance {
         double totalTime = 0;
         int totalSuccess = 0;
         for (int i = 0; i < threadCount; i++) {
-            System.out.println("Thread: " + i + " Success: " +
-                    runnables[i].getSuccess());
+            System.out.println("Thread: " + i + " Success: " + runnables[i].getSuccess());
             totalSuccess = totalSuccess + runnables[i].getSuccess();
             totalTime = totalTime + runnables[i].getTime();
         }
 
-        System.out.println("Average time per request (user): " +
-                totalTime/(threadCount * requestCount));
-        System.out.println("Average time per request (wall): " +
-                wallTime/(threadCount * requestCount));
+        System.out.println("Average time per request (user): " + totalTime / (threadCount * requestCount));
+        System.out.println("Average time per request (wall): " + wallTime / (threadCount * requestCount));
 
-        Assert.assertEquals(((long)requestCount) * threadCount, totalSuccess);
+        Assert.assertEquals(((long) requestCount) * threadCount, totalSuccess);
     }
 
     @Before
@@ -120,6 +123,15 @@ public class TesterDigestAuthenticatorPerformance {
         context.setName(CONTEXT_PATH);
         context.setRealm(realm);
 
+        Host host = new StandardHost();
+        context.setParent(host);
+
+        Engine engine = new StandardEngine();
+        host.setParent(engine);
+
+        Service service = new StandardService();
+        engine.setService(service);
+
         // Configure the Login config
         LoginConfig config = new LoginConfig();
         config.setRealmName(REALM);
@@ -148,16 +160,14 @@ public class TesterDigestAuthenticatorPerformance {
         private static final String A1 = USER + ":" + REALM + ":" + PWD;
         private static final String A2 = METHOD + ":" + CONTEXT_PATH + URI;
 
-        private static final String DIGEST_A1 = HexUtils.toHexString(
-                ConcurrentMessageDigest.digest("MD5", A1.getBytes()));
-        private static final String DIGEST_A2 = HexUtils.toHexString(
-                ConcurrentMessageDigest.digest("MD5", A2.getBytes()));
-
+        private static final String DIGEST_A1 =
+                HexUtils.toHexString(ConcurrentMessageDigest.digest("MD5", A1.getBytes(StandardCharsets.UTF_8)));
+        private static final String DIGEST_A2 =
+                HexUtils.toHexString(ConcurrentMessageDigest.digest("MD5", A2.getBytes(StandardCharsets.UTF_8)));
 
 
         // All init code should be in here. run() needs to be quick
-        TesterRunnable(DigestAuthenticator authenticator,
-                String nonce, int requestCount) throws Exception {
+        TesterRunnable(DigestAuthenticator authenticator, String nonce, int requestCount) throws Exception {
             this.authenticator = authenticator;
             this.nonce = nonce;
             this.requestCount = requestCount;
@@ -196,15 +206,13 @@ public class TesterDigestAuthenticatorPerformance {
 
         private String buildDigestResponse(String nonce) {
 
-            String ncString = String.format("%1$08x",
-                    Integer.valueOf(nonceCount.incrementAndGet()));
+            String ncString = String.format("%1$08x", Integer.valueOf(nonceCount.incrementAndGet()));
             String cnonce = "cnonce";
 
-            String response = DIGEST_A1 + ":" + nonce + ":" + ncString + ":" +
-                    cnonce + ":" + QOP + ":" + DIGEST_A2;
+            String response = DIGEST_A1 + ":" + nonce + ":" + ncString + ":" + cnonce + ":" + QOP + ":" + DIGEST_A2;
 
-            String md5response = HexUtils.toHexString(
-                    ConcurrentMessageDigest.digest("MD5", response.getBytes()));
+            String md5response = HexUtils
+                    .toHexString(ConcurrentMessageDigest.digest("MD5", response.getBytes(StandardCharsets.UTF_8)));
 
             StringBuilder auth = new StringBuilder();
             auth.append("Digest username=\"");
@@ -237,6 +245,10 @@ public class TesterDigestAuthenticatorPerformance {
 
         private String authHeader = null;
 
+        TesterDigestRequest() {
+            super(null);
+        }
+
         @Override
         public String getRemoteAddr() {
             return "127.0.0.1";
