@@ -57,8 +57,7 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
     /*
      * Test whether size limited uploads correctly handle connection draining.
      */
-    public Exception doAbortedUploadTest(AbortedUploadClient client, boolean limited,
-                            boolean swallow) {
+    public Exception doAbortedUploadTest(AbortedUploadClient client, boolean limited, boolean swallow) {
         Exception ex = client.doRequest(limited, swallow);
         if (log.isDebugEnabled()) {
             log.debug("Response line: " + client.getResponseLine());
@@ -75,8 +74,7 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
     /*
      * Test whether aborted POST correctly handle connection draining.
      */
-    public Exception doAbortedPOSTTest(AbortedPOSTClient client, int status,
-                            boolean swallow) {
+    public Exception doAbortedPOSTTest(AbortedPOSTClient client, int status, boolean swallow) {
         Exception ex = client.doRequest(status, swallow);
         if (log.isDebugEnabled()) {
             log.debug("Response line: " + client.getResponseLine());
@@ -95,10 +93,8 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
         log.info("Unlimited, swallow enabled");
         AbortedUploadClient client = new AbortedUploadClient();
         Exception ex = doAbortedUploadTest(client, false, true);
-        Assert.assertNull("Unlimited upload with swallow enabled generates client exception",
-                   ex);
-        Assert.assertTrue("Unlimited upload with swallow enabled returns error status code",
-                   client.isResponse200());
+        Assert.assertNull("Unlimited upload with swallow enabled generates client exception", ex);
+        Assert.assertTrue("Unlimited upload with swallow enabled returns error status code", client.isResponse200());
         client.reset();
     }
 
@@ -107,10 +103,8 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
         log.info("Unlimited, swallow disabled");
         AbortedUploadClient client = new AbortedUploadClient();
         Exception ex = doAbortedUploadTest(client, false, false);
-        Assert.assertNull("Unlimited upload with swallow disabled generates client exception",
-                   ex);
-        Assert.assertTrue("Unlimited upload with swallow disabled returns error status code",
-                   client.isResponse200());
+        Assert.assertNull("Unlimited upload with swallow disabled generates client exception", ex);
+        Assert.assertTrue("Unlimited upload with swallow disabled returns error status code", client.isResponse200());
         client.reset();
     }
 
@@ -119,10 +113,8 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
         log.info("Limited, swallow enabled");
         AbortedUploadClient client = new AbortedUploadClient();
         Exception ex = doAbortedUploadTest(client, true, true);
-        Assert.assertNull("Limited upload with swallow enabled generates client exception",
-                   ex);
-        Assert.assertTrue("Limited upload with swallow enabled returns non-500 status code",
-                   client.isResponse500());
+        Assert.assertNull("Limited upload with swallow enabled generates client exception", ex);
+        Assert.assertTrue("Limited upload with swallow enabled returns non-500 status code", client.isResponse500());
         client.reset();
     }
 
@@ -131,8 +123,8 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
         log.info("Limited, swallow disabled");
         AbortedUploadClient client = new AbortedUploadClient();
         Exception ex = doAbortedUploadTest(client, true, false);
-        assertThat("Limited upload with swallow disabled does not generate client exception",
-                   ex, instanceOf(java.net.SocketException.class));
+        assertThat("Limited upload with swallow disabled does not generate client exception", ex,
+                instanceOf(java.net.SocketException.class));
         client.reset();
     }
 
@@ -141,10 +133,8 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
         log.info("Aborted (OK), swallow enabled");
         AbortedPOSTClient client = new AbortedPOSTClient();
         Exception ex = doAbortedPOSTTest(client, HttpServletResponse.SC_OK, true);
-        Assert.assertNull("Unlimited upload with swallow enabled generates client exception",
-                   ex);
-        Assert.assertTrue("Unlimited upload with swallow enabled returns error status code",
-                   client.isResponse200());
+        Assert.assertNull("Unlimited upload with swallow enabled generates client exception", ex);
+        Assert.assertTrue("Unlimited upload with swallow enabled returns error status code", client.isResponse200());
         client.reset();
     }
 
@@ -153,10 +143,8 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
         log.info("Aborted (OK), swallow disabled");
         AbortedPOSTClient client = new AbortedPOSTClient();
         Exception ex = doAbortedPOSTTest(client, HttpServletResponse.SC_OK, false);
-        Assert.assertNull("Unlimited upload with swallow disabled generates client exception",
-                   ex);
-        Assert.assertTrue("Unlimited upload with swallow disabled returns error status code",
-                   client.isResponse200());
+        Assert.assertNull("Unlimited upload with swallow disabled generates client exception", ex);
+        Assert.assertTrue("Unlimited upload with swallow disabled returns error status code", client.isResponse200());
         client.reset();
     }
 
@@ -165,10 +153,8 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
         log.info("Aborted (413), swallow enabled");
         AbortedPOSTClient client = new AbortedPOSTClient();
         Exception ex = doAbortedPOSTTest(client, HttpServletResponse.SC_REQUEST_ENTITY_TOO_LARGE, true);
-        Assert.assertNull("Limited upload with swallow enabled generates client exception",
-                   ex);
-        Assert.assertTrue("Limited upload with swallow enabled returns error status code",
-                   client.isResponse413());
+        Assert.assertNull("Limited upload with swallow enabled generates client exception", ex);
+        Assert.assertTrue("Limited upload with swallow enabled returns error status code", client.isResponse413());
         client.reset();
     }
 
@@ -177,8 +163,8 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
         log.info("Aborted (413), swallow disabled");
         AbortedPOSTClient client = new AbortedPOSTClient();
         Exception ex = doAbortedPOSTTest(client, HttpServletResponse.SC_REQUEST_ENTITY_TOO_LARGE, false);
-        assertThat("Limited upload with swallow disabled does not generate client exception",
-                ex, instanceOf(java.net.SocketException.class));
+        assertThat("Limited upload with swallow disabled does not generate client exception", ex,
+                instanceOf(java.net.SocketException.class));
         client.reset();
     }
 
@@ -188,8 +174,7 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
         private static final long serialVersionUID = 1L;
 
         @Override
-        protected void doPost(HttpServletRequest req, HttpServletResponse resp)
-                throws ServletException, IOException {
+        protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
             PrintWriter out = resp.getWriter();
             resp.setContentType("text/plain");
             resp.setCharacterEncoding("UTF-8");
@@ -203,10 +188,8 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
                     log.debug("Count: " + c.size());
                     sb.append("Count: " + c.size() + "\n");
                     for (Part p : c) {
-                        log.debug("Name: " + p.getName() + ", Size: "
-                                + p.getSize());
-                        sb.append("Name: " + p.getName() + ", Size: "
-                                + p.getSize() + "\n");
+                        log.debug("Name: " + p.getName() + ", Size: " + p.getSize());
+                        sb.append("Name: " + p.getName() + ", Size: " + p.getSize() + "\n");
                     }
                 }
             } catch (IllegalStateException ex) {
@@ -236,20 +219,17 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
 
         private Context context;
 
-        private synchronized void init(boolean limited, boolean swallow)
-                throws Exception {
+        private synchronized void init(boolean limited, boolean swallow) throws Exception {
 
             Tomcat tomcat = getTomcatInstance();
             context = tomcat.addContext("", TEMP_DIR);
             Wrapper w;
-            w = Tomcat.addServlet(context, servletName,
-                                  new AbortedUploadServlet());
+            w = Tomcat.addServlet(context, servletName, new AbortedUploadServlet());
             // Tomcat.addServlet does not respect annotations, so we have
             // to set our own MultipartConfigElement.
             // Choose upload file size limit.
             if (limited) {
-                w.setMultipartConfigElement(new MultipartConfigElement("",
-                        limitSize, -1, -1));
+                w.setMultipartConfigElement(new MultipartConfigElement("", limitSize, -1, -1));
             } else {
                 w.setMultipartConfigElement(new MultipartConfigElement(""));
             }
@@ -294,16 +274,19 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
                 sb.append(CRLF);
 
                 // Re-encode the content so that bytes = characters
-                String content = new String(sb.toString().getBytes("UTF-8"),
-                        "ASCII");
+                String content = new String(sb.toString().getBytes("UTF-8"), "ASCII");
 
-                request = new String[] { "POST http://localhost:" + getPort() + URI + " HTTP/1.1" + CRLF
+                // @formatter:off
+                request = new String[] {
+                        "POST http://localhost:" + getPort() + URI + " HTTP/1.1" + CRLF
                         + "Host: localhost:" + getPort() + CRLF
                         + "Connection: close" + CRLF
                         + "Content-Type: multipart/form-data; boundary=" + boundary + CRLF
                         + "Content-Length: " + content.length() + CRLF
                         + CRLF
-                        + content + CRLF };
+                        + content + CRLF
+                        };
+                // @formatter:on
 
                 setRequest(request);
                 processRequest(); // blocks until response has been read
@@ -333,8 +316,7 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
         }
 
         @Override
-        protected void doPost(HttpServletRequest req, HttpServletResponse resp)
-                throws ServletException, IOException {
+        protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
             resp.setContentType("text/plain");
             resp.setCharacterEncoding("UTF-8");
             resp.setStatus(status);
@@ -356,8 +338,7 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
 
         private Context context;
 
-        private synchronized void init(int status, boolean swallow)
-                throws Exception {
+        private synchronized void init(int status, boolean swallow) throws Exception {
 
             Tomcat tomcat = getTomcatInstance();
             context = tomcat.addContext("", TEMP_DIR);
@@ -390,12 +371,15 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
 
                 String content = new String(body);
 
-                request = new String[] { "POST http://localhost:" + getPort() + URI + " HTTP/1.1" + CRLF
+                // @formatter:off
+                request = new String[] {
+                        "POST http://localhost:" + getPort() + URI + " HTTP/1.1" + CRLF
                         + "Host: localhost:" + getPort() + CRLF
                         + "Connection: close" + CRLF
                         + "Content-Length: " + content.length() + CRLF
                         + CRLF
                         + content + CRLF };
+                // @formatter:on
 
                 setRequest(request);
                 processRequest(); // blocks until response has been read
@@ -444,8 +428,7 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
         String responseLine = null;
 
         try (Socket conn = new Socket("localhost", getPort())) {
-            Writer writer = new OutputStreamWriter(
-                    conn.getOutputStream(), StandardCharsets.US_ASCII);
+            Writer writer = new OutputStreamWriter(conn.getOutputStream(), StandardCharsets.US_ASCII);
             writer.write("PUT /does-not-exist HTTP/1.1\r\n");
             writer.write("Host: any\r\n");
             writer.write("Transfer-encoding: chunked\r\n");
@@ -454,7 +437,7 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
             // Smarter than the typical client. Attempts to read the response
             // even if the request is not fully written.
             try {
-                // Write (or try to write) 16MB
+                // Write (or try to write) 16 MiB
                 for (int i = 0; i < 1024 * 1024; i++) {
                     writer.write("10\r\n");
                     writer.write("0123456789ABCDEF\r\n");
@@ -464,12 +447,12 @@ public class TestSwallowAbortedUploads extends TomcatBaseTest {
             }
 
             try {
-                BufferedReader reader = new BufferedReader(new InputStreamReader(
-                        conn.getInputStream(), StandardCharsets.US_ASCII));
+                BufferedReader reader =
+                        new BufferedReader(new InputStreamReader(conn.getInputStream(), StandardCharsets.US_ASCII));
 
                 responseLine = reader.readLine();
-            } catch (IOException e) {
-                readEx = e;
+            } catch (IOException ioe) {
+                readEx = ioe;
             }
         }
 
