@@ -37,7 +37,7 @@ public class TestMapper extends LoggingBaseTest {
 
     protected Mapper mapper;
 
-    private HashMap<String, Host> hostMap = new HashMap<>();
+    private HashMap<String,Host> hostMap = new HashMap<>();
 
     private synchronized Host createHost(String name) {
         Host host = hostMap.get(name);
@@ -82,6 +82,9 @@ public class TestMapper extends LoggingBaseTest {
         mapper.addHost("zzzuyopjvewpovewjhfewoih", new String[0], createHost("blah12"));
         mapper.addHost("xxxxgqwiwoih", new String[0], createHost("blah13"));
         mapper.addHost("qwigqwiwoih", new String[0], createHost("blah14"));
+        mapper.addHost("qwerty.net", new String[0], createHost("blah15"));
+        mapper.addHost("*.net", new String[0], createHost("blah16"));
+        mapper.addHost("zzz.com", new String[0], createHost("blah17"));
         mapper.addHostAlias("iowejoiejfoiew", "iowejoiejfoiew_alias");
 
         mapper.setDefaultHostName("ylwrehirkuewh");
@@ -91,44 +94,32 @@ public class TestMapper extends LoggingBaseTest {
         welcomes[1] = "bobou";
 
         Host host = createHost("blah7");
-        mapper.addContextVersion("iowejoiejfoiew", host, "",
-                "0", createContext("context0"), new String[0], null, null);
-        mapper.addContextVersion("iowejoiejfoiew", host, "/foo",
-                "0", createContext("context1"), new String[0], null, null);
-        mapper.addContextVersion("iowejoiejfoiew", host, "/foo/bar",
-                "0", createContext("context2"), welcomes, null, null);
+        mapper.addContextVersion("iowejoiejfoiew", host, "", "0", createContext("context0"), new String[0], null, null);
+        mapper.addContextVersion("iowejoiejfoiew", host, "/foo", "0", createContext("context1"), new String[0], null,
+                null);
+        mapper.addContextVersion("iowejoiejfoiew", host, "/foo/bar", "0", createContext("context2"), welcomes, null,
+                null);
 
-        mapper.addWrappers("iowejoiejfoiew", "/foo", "0", Arrays
-                .asList(new WrapperMappingInfo[] {
-                        new WrapperMappingInfo("/",
-                                createWrapper("context1-defaultWrapper"), false, false) }));
-        mapper.addWrappers("iowejoiejfoiew", "/foo/bar", "0", Arrays
-                .asList(new WrapperMappingInfo[] {
-                        new WrapperMappingInfo("/fo/*",
-                                createWrapper("wrapper0"), false, false),
-                        new WrapperMappingInfo("/", createWrapper("wrapper1"),
-                                false, false),
-                        new WrapperMappingInfo("/blh",
-                                createWrapper("wrapper2"), false, false),
-                        new WrapperMappingInfo("*.jsp",
-                                createWrapper("wrapper3"), false, false),
-                        new WrapperMappingInfo("/blah/bou/*",
-                                createWrapper("wrapper4"), false, false),
-                        new WrapperMappingInfo("/blah/bobou/*",
-                                createWrapper("wrapper5"), false, false),
-                        new WrapperMappingInfo("*.htm",
-                                createWrapper("wrapper6"), false, false) }));
+        mapper.addWrappers("iowejoiejfoiew", "/foo", "0", Arrays.asList(new WrapperMappingInfo[] {
+                new WrapperMappingInfo("/", createWrapper("context1-defaultWrapper"), false, false) }));
+        mapper.addWrappers("iowejoiejfoiew", "/foo/bar", "0",
+                Arrays.asList(new WrapperMappingInfo[] {
+                        new WrapperMappingInfo("/fo/*", createWrapper("wrapper0"), false, false),
+                        new WrapperMappingInfo("/", createWrapper("wrapper1"), false, false),
+                        new WrapperMappingInfo("/blh", createWrapper("wrapper2"), false, false),
+                        new WrapperMappingInfo("*.jsp", createWrapper("wrapper3"), false, false),
+                        new WrapperMappingInfo("/blah/bou/*", createWrapper("wrapper4"), false, false),
+                        new WrapperMappingInfo("/blah/bobou/*", createWrapper("wrapper5"), false, false),
+                        new WrapperMappingInfo("*.htm", createWrapper("wrapper6"), false, false) }));
 
-        mapper.addContextVersion(
-                "iowejoiejfoiew",
-                host,
-                "/foo/bar/bla",
-                "0",
-                createContext("context3"),
-                new String[0],
-                null,
-                Arrays.asList(new WrapperMappingInfo[] { new WrapperMappingInfo(
-                        "/bobou/*", createWrapper("wrapper7"), false, false) }));
+        mapper.addContextVersion("iowejoiejfoiew", host, "/foo/bar/bla", "0", createContext("context3"), new String[0],
+                null, Arrays.asList(new WrapperMappingInfo[] {
+                        new WrapperMappingInfo("/bobou/*", createWrapper("wrapper7"), false, false) }));
+
+        host = createHost("blah16");
+        mapper.addContextVersion("*.net", host, "", "0", createContext("context4"), new String[0], null, null);
+        mapper.addWrappers("*.net", "", "0", Arrays.asList(new WrapperMappingInfo[] {
+                new WrapperMappingInfo("/", createWrapper("context4-defaultWrapper"), false, false) }));
     }
 
     @Test
@@ -147,14 +138,14 @@ public class TestMapper extends LoggingBaseTest {
         mapper.addHostAlias("iowejoiejfoiew", "iowejoiejfoiew_alias");
 
         // Check we have the right number
-        // (added 16 including one host alias. Three duplicates do not increase the count.)
-        Assert.assertEquals(16, mapper.hosts.length);
+        // (added 17 including one host alias. Three duplicates do not increase the count.)
+        Assert.assertEquals(19, mapper.hosts.length);
 
         // Make sure adding a duplicate *does not* overwrite
-        final int iowPos = 3;
+        final int iowPos = 4;
         Assert.assertEquals("blah7", mapper.hosts[iowPos].object.getName());
 
-        final int qwigPos = 8;
+        final int qwigPos = 10;
         Assert.assertEquals("blah14", mapper.hosts[qwigPos].object.getName());
 
         // Check for alphabetical order of host names
@@ -179,38 +170,34 @@ public class TestMapper extends LoggingBaseTest {
         Host hostZ = createHost("zzzz");
         Context contextZ = createContext("contextZ");
 
-        Assert.assertEquals(16, mapper.hosts.length);
-        mapper.addContextVersion("zzzz", hostZ, "/", "", contextZ, null, null,
-                null);
-        Assert.assertEquals(17, mapper.hosts.length);
-
-        mapper.addHost("zzzz", new String[] { "zzzz_alias1", "zzzz_alias2" },
-                hostZ);
         Assert.assertEquals(19, mapper.hosts.length);
+        mapper.addContextVersion("zzzz", hostZ, "/", "", contextZ, null, null, null);
+        Assert.assertEquals(20, mapper.hosts.length);
 
-        Assert.assertEquals("zzzz", mapper.hosts[16].name);
-        Assert.assertEquals("zzzz_alias1", mapper.hosts[17].name);
-        Assert.assertEquals("zzzz_alias2", mapper.hosts[18].name);
-        Assert.assertEquals(2, mapper.hosts[16].getAliases().size());
-        Assert.assertSame(contextZ,
-                mapper.hosts[16].contextList.contexts[0].versions[0].object);
-        Assert.assertSame(contextZ,
-                mapper.hosts[18].contextList.contexts[0].versions[0].object);
+        mapper.addHost("zzzz", new String[] { "zzzz_alias1", "zzzz_alias2" }, hostZ);
+        Assert.assertEquals(22, mapper.hosts.length);
+
+        Assert.assertEquals("zzzz", mapper.hosts[19].name);
+        Assert.assertEquals("zzzz_alias1", mapper.hosts[20].name);
+        Assert.assertEquals("zzzz_alias2", mapper.hosts[21].name);
+        Assert.assertEquals(2, mapper.hosts[19].getAliases().size());
+        Assert.assertSame(contextZ, mapper.hosts[19].contextList.contexts[0].versions[0].object);
+        Assert.assertSame(contextZ, mapper.hosts[21].contextList.contexts[0].versions[0].object);
     }
 
     @Test
     public void testRemoveHost() {
-        Assert.assertEquals(16, mapper.hosts.length);
+        Assert.assertEquals(19, mapper.hosts.length);
         mapper.removeHostAlias("iowejoiejfoiew");
         mapper.removeHost("iowejoiejfoiew_alias");
-        Assert.assertEquals(16, mapper.hosts.length); // No change
+        Assert.assertEquals(19, mapper.hosts.length); // No change
         mapper.removeHostAlias("iowejoiejfoiew_alias");
-        Assert.assertEquals(15, mapper.hosts.length); // Removed
+        Assert.assertEquals(18, mapper.hosts.length); // Removed
 
         mapper.addHostAlias("iowejoiejfoiew", "iowejoiejfoiew_alias");
-        Assert.assertEquals(16, mapper.hosts.length);
+        Assert.assertEquals(19, mapper.hosts.length);
 
-        final int iowPos = 3;
+        final int iowPos = 4;
         Mapper.MappedHost hostMapping = mapper.hosts[iowPos];
         Mapper.MappedHost aliasMapping = mapper.hosts[iowPos + 1];
         Assert.assertEquals("iowejoiejfoiew_alias", aliasMapping.name);
@@ -223,17 +210,20 @@ public class TestMapper extends LoggingBaseTest {
         Assert.assertSame(hostMapping, aliasMapping.getRealHost());
 
         mapper.removeHost("iowejoiejfoiew");
-        Assert.assertEquals(14, mapper.hosts.length); // Both host and alias removed
+        Assert.assertEquals(17, mapper.hosts.length); // Both host and alias removed
         for (Mapper.MappedHost host : mapper.hosts) {
             Assert.assertTrue(host.name, !host.name.startsWith("iowejoiejfoiew"));
         }
     }
 
     @Test
+    @SuppressWarnings("deprecation") // contextPath
     public void testMap() throws Exception {
         MappingData mappingData = new MappingData();
         MessageBytes host = MessageBytes.newInstance();
         host.setString("iowejoiejfoiew");
+        MessageBytes wildcard = MessageBytes.newInstance();
+        wildcard.setString("foo.net");
         MessageBytes alias = MessageBytes.newInstance();
         alias.setString("iowejoiejfoiew_alias");
         MessageBytes uri = MessageBytes.newInstance();
@@ -264,6 +254,20 @@ public class TestMapper extends LoggingBaseTest {
         Assert.assertEquals("/foo", mappingData.pathInfo.toString());
         Assert.assertTrue(mappingData.redirectPath.isNull());
 
+        mappingData.recycle();
+        uri.recycle();
+        uri.setString("/foo/bar/bla/bobou/foo");
+        uri.toChars();
+        uri.getCharChunk().setLimit(-1);
+        mapper.map(wildcard, uri, null, mappingData);
+        Assert.assertEquals("blah16", mappingData.host.getName());
+        Assert.assertEquals("context4", mappingData.context.getName());
+        Assert.assertEquals("context4-defaultWrapper", mappingData.wrapper.getName());
+        Assert.assertEquals("", mappingData.contextPath.toString());
+        Assert.assertEquals("/foo/bar/bla/bobou/foo", mappingData.wrapperPath.toString());
+        Assert.assertTrue(mappingData.pathInfo.isNull());
+        Assert.assertTrue(mappingData.redirectPath.isNull());
+
         mappingData.recycle();
         uri.setString("/foo/bar/bla/bobou/foo");
         uri.toChars();
@@ -281,7 +285,7 @@ public class TestMapper extends LoggingBaseTest {
     @Test
     public void testAddRemoveContextVersion() throws Exception {
         final String hostName = "iowejoiejfoiew";
-        final int iowPos = 3;
+        final int iowPos = 4;
         final String contextPath = "/foo/bar";
         final int contextPos = 2;
 
@@ -315,16 +319,9 @@ public class TestMapper extends LoggingBaseTest {
         Assert.assertEquals("wrapper5", mappingData.wrapper.getName());
 
         Context newContext = createContext("newContext");
-        mapper.addContextVersion(
-                hostName,
-                oldHost,
-                contextPath,
-                "1",
-                newContext,
-                null,
-                null,
-                Arrays.asList(new WrapperMappingInfo[] { new WrapperMappingInfo(
-                        "/", createWrapper("newContext-default"), false, false) }));
+        mapper.addContextVersion(hostName, oldHost, contextPath, "1", newContext, null, null,
+                Arrays.asList(new WrapperMappingInfo[] {
+                        new WrapperMappingInfo("/", createWrapper("newContext-default"), false, false) }));
 
         Assert.assertEquals(2, mappedContext.versions.length);
         Assert.assertEquals("0", mappedContext.versions[0].name);
@@ -361,16 +358,9 @@ public class TestMapper extends LoggingBaseTest {
         mapper.map(oldContext1, uriMB, mappingData);
         Assert.assertEquals("context1-defaultWrapper", mappingData.wrapper.getName());
 
-        mapper.addContextVersion(
-                hostName,
-                oldHost,
-                contextPath,
-                "0",
-                newContext,
-                null,
-                null,
-                Arrays.asList(new WrapperMappingInfo[] { new WrapperMappingInfo(
-                        "/", createWrapper("newContext-defaultWrapper2"), false, false) }));
+        mapper.addContextVersion(hostName, oldHost, contextPath, "0", newContext, null, null,
+                Arrays.asList(new WrapperMappingInfo[] {
+                        new WrapperMappingInfo("/", createWrapper("newContext-defaultWrapper2"), false, false) }));
         mappedContext = mappedHost.contextList.contexts[contextPos];
 
         Assert.assertEquals(contextPath, mappedContext.name);
@@ -388,7 +378,7 @@ public class TestMapper extends LoggingBaseTest {
     @Test
     public void testReloadContextVersion() throws Exception {
         final String hostName = "iowejoiejfoiew";
-        final int iowPos = 3;
+        final int iowPos = 4;
         final String contextPath = "/foo/bar";
         final int contextPos = 2;
 
@@ -438,16 +428,9 @@ public class TestMapper extends LoggingBaseTest {
 
         // Re-add the same context, but different list of wrappers
         // This is what happens when context reload completes
-        mapper.addContextVersion(
-                hostName,
-                oldHost,
-                contextPath,
-                "0",
-                oldContext,
-                null,
-                null,
-                Arrays.asList(new WrapperMappingInfo[] { new WrapperMappingInfo(
-                        "/", createWrapper("newDefaultWrapper"), false, false) }));
+        mapper.addContextVersion(hostName, oldHost, contextPath, "0", oldContext, null, null,
+                Arrays.asList(new WrapperMappingInfo[] {
+                        new WrapperMappingInfo("/", createWrapper("newDefaultWrapper"), false, false) }));
 
         mappedContext = mappedHost.contextList.contexts[contextPos];
         Assert.assertEquals(contextPath, mappedContext.name);
@@ -465,6 +448,7 @@ public class TestMapper extends LoggingBaseTest {
     }
 
     @Test
+    @SuppressWarnings("deprecation") // contextPath
     public void testContextListConcurrencyBug56653() throws Exception {
         final Host host = createHost("localhost");
         final Context contextRoot = createContext("ROOT");
@@ -476,27 +460,20 @@ public class TestMapper extends LoggingBaseTest {
         mapper.addHost("localhost", new String[] { "alias" }, host);
         mapper.setDefaultHostName("localhost");
 
-        mapper.addContextVersion("localhost", host, "", "0", contextRoot,
-                new String[0], null, null);
-        mapper.addContextVersion("localhost", host, "/foo", "0", context1,
-                new String[0], null, null);
-        mapper.addContextVersion("localhost", host, "/foo/bar", "0", context2,
-                new String[0], null, null);
-        mapper.addContextVersion("localhost", host, "/foo/bar/bla", "0",
-                context3, new String[0], null, null);
-        mapper.addContextVersion("localhost", host, "/foo/bar/bla/baz", "0",
-                context4, new String[0], null, null);
+        mapper.addContextVersion("localhost", host, "", "0", contextRoot, new String[0], null, null);
+        mapper.addContextVersion("localhost", host, "/foo", "0", context1, new String[0], null, null);
+        mapper.addContextVersion("localhost", host, "/foo/bar", "0", context2, new String[0], null, null);
+        mapper.addContextVersion("localhost", host, "/foo/bar/bla", "0", context3, new String[0], null, null);
+        mapper.addContextVersion("localhost", host, "/foo/bar/bla/baz", "0", context4, new String[0], null, null);
 
         final AtomicBoolean running = new AtomicBoolean(true);
         Thread t = new Thread() {
             @Override
             public void run() {
                 for (int i = 0; i < 100000; i++) {
-                    mapper.removeContextVersion(context4, "localhost",
-                            "/foo/bar/bla/baz", "0");
-                    mapper.addContextVersion("localhost", host,
-                            "/foo/bar/bla/baz", "0", context4, new String[0],
-                            null, null);
+                    mapper.removeContextVersion(context4, "localhost", "/foo/bar/bla/baz", "0");
+                    mapper.addContextVersion("localhost", host, "/foo/bar/bla/baz", "0", context4, new String[0], null,
+                            null);
                 }
                 running.set(false);
             }
@@ -532,4 +509,47 @@ public class TestMapper extends LoggingBaseTest {
             Assert.assertEquals("/foo/bar/bla", mappingData.contextPath.toString());
         }
     }
+
+
+    @Test
+    public void testCompareIgnoreCase() throws Exception {
+
+        Mapper mapper = new Mapper();
+
+        mapper.addHost("aaa", new String[0], createHost("a3"));
+        mapper.addHost("aaaa", new String[0], createHost("a4"));
+        mapper.addHost("aaaaa", new String[0], createHost("a5"));
+        mapper.addHost("aaaaaa", new String[0], createHost("a6"));
+        mapper.addHost("aaaaaaa", new String[0], createHost("a7"));
+
+        mapper.setDefaultHostName("aaa");
+
+        mapper.addContextVersion("aaa", createHost("a3"), "", "0", createContext("c3"), new String[0], null, null);
+        mapper.addContextVersion("aaaa", createHost("a4"), "", "0", createContext("c4"), new String[0], null, null);
+        mapper.addContextVersion("aaaaa", createHost("a5"), "", "0", createContext("c5"), new String[0], null, null);
+        mapper.addContextVersion("aaaaaa", createHost("a6"), "", "0", createContext("c6"), new String[0], null, null);
+        mapper.addContextVersion("aaaaaaa", createHost("a7"), "", "0", createContext("c7"), new String[0], null, null);
+
+        mapper.addWrappers("aaa", "", "0", Arrays.asList(
+                new WrapperMappingInfo[] { new WrapperMappingInfo("/", createWrapper("c3-default"), false, false) }));
+        mapper.addWrappers("aaaa", "", "0", Arrays.asList(
+                new WrapperMappingInfo[] { new WrapperMappingInfo("/", createWrapper("c4-default"), false, false) }));
+        mapper.addWrappers("aaaaa", "", "0", Arrays.asList(
+                new WrapperMappingInfo[] { new WrapperMappingInfo("/", createWrapper("c5-default"), false, false) }));
+        mapper.addWrappers("aaaaaa", "", "0", Arrays.asList(
+                new WrapperMappingInfo[] { new WrapperMappingInfo("/", createWrapper("c6-default"), false, false) }));
+        mapper.addWrappers("aaaaaaa", "", "0", Arrays.asList(
+                new WrapperMappingInfo[] { new WrapperMappingInfo("/", createWrapper("c7-default"), false, false) }));
+
+        MappingData mappingData = new MappingData();
+        MessageBytes hostMB = MessageBytes.newInstance();
+        hostMB.setString("aaaa");
+        MessageBytes uriMB = MessageBytes.newInstance();
+        char[] uri = "/index.html".toCharArray();
+        uriMB.setChars(uri, 0, uri.length);
+
+        mapper.map(hostMB, uriMB, null, mappingData);
+
+        Assert.assertEquals("a4", mappingData.host.getName());
+    }
 }
