@@ -44,6 +44,7 @@ import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
 import org.junit.Assert;
+import org.junit.Assume;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
@@ -62,6 +63,7 @@ import org.apache.catalina.startup.Tomcat;
 import org.apache.catalina.startup.TomcatBaseTest;
 import org.apache.catalina.valves.ValveBase;
 import org.apache.tomcat.util.buf.ByteChunk;
+import org.apache.tomcat.util.net.openssl.OpenSSLStatus;
 import org.apache.tomcat.websocket.server.WsContextListener;
 
 /**
@@ -146,6 +148,11 @@ public class TestSsl extends TomcatBaseTest {
         TesterSupport.configureSSLImplementation(tomcat, sslImplementationName, useOpenSSL);
 
         tomcat.start();
+
+        Assume.assumeFalse("BoringSSL and LibreSSL return no session id",
+                TesterSupport.isOpenSSLVariant(sslImplementationName, OpenSSLStatus.Name.BORINGSSL)
+                    || TesterSupport.isOpenSSLVariant(sslImplementationName, OpenSSLStatus.Name.LIBRESSL));
+
         getUrl("https://localhost:" + getPort() + "/examples/servlets/servlet/HelloWorldExample");
         // SSL is the only source for the requested session ID, and SessionTrackingMode.SSL is set on examples
         Assert.assertNotNull(sessionId);
@@ -327,7 +334,7 @@ public class TestSsl extends TomcatBaseTest {
             if (!renegotiationSupported) {
                 Assert.fail("Renegotiation started when it should have failed");
             }
-        } catch (IOException e) {
+        } catch (IOException ioe) {
             if (renegotiationSupported) {
                 Assert.fail("Renegotiation failed when it should be supported");
             }
